@@ -73,9 +73,9 @@ public:
 };
 
 /////////////////////////////////////////////////////////////////////////////
-// color_traits<boost::uint8_t>
+// color_traits<boost::uint_t>
 
-/// Specialization of color_traits for uint8_t samples
+/// Specialization of color_traits for 8 bit samples
 template<>
 class color_traits<boost::uint8_t>
 {
@@ -104,6 +104,38 @@ public:
 	static inline sample_type convert(const double Sample) { return static_cast<sample_type>(clamp(Sample, double(0), double(1)) * maximum()); }
 };
 
+/////////////////////////////////////////////////////////////////////////////
+// color_traits<boost::uint16_t>
+
+/// Specialization of color_traits for 16 bit samples
+template<>
+class color_traits<boost::uint16_t>
+{
+public:
+	/// Defines the sample type for this specialization
+	typedef boost::uint16_t sample_type;
+
+	/// Returns the minimum value of a sample
+	static inline sample_type minimum() { return 0; }
+	/// Returns the maximum value of a sample
+	static inline sample_type maximum() { return 65535; }
+	/// Returns the sample value corresponding to transparent alpha
+	static inline sample_type transparent() { return minimum(); }
+	/// Returns the sample value corresponding to opaque alpha
+	static inline sample_type opaque() { return maximum(); }
+	/// Inverts the value of a sample
+	static inline sample_type invert(const sample_type& Sample) { return maximum() - Sample; }
+
+	/// Converts a sample from a uint16_t
+	static inline sample_type convert(const sample_type Sample) { return Sample; }
+	/// Converts a sample from a half
+	static inline sample_type convert(const half Sample) { return static_cast<sample_type>(clamp(Sample, half(0), half(1)) * maximum()); }
+	/// Converts a sample from a float
+	static inline sample_type convert(const float Sample) { return static_cast<sample_type>(clamp(Sample, float(0), float(1)) * maximum()); }
+	/// Converts a sample from a double
+	static inline sample_type convert(const double Sample) { return static_cast<sample_type>(clamp(Sample, double(0), double(1)) * maximum()); }
+};
+
 /////////////////////////////////////////////////////////////////////////////
 // color_traits<half>
 
@@ -127,13 +159,15 @@ public:
 	static inline sample_type invert(const sample_type& Sample) { return maximum() - Sample; }
 
 	/// Converts a sample from a uint8_t
-	static inline sample_type convert(const boost::uint8_t Sample) { return static_cast<sample_type>(Sample) / 255.0; }
+	static inline sample_type convert(const boost::uint8_t Sample) { return static_cast<sample_type>(Sample) / 255.0f; }
+	/// Converts a sample from a uint16_t
+	static inline sample_type convert(const boost::uint16_t Sample) { return static_cast<sample_type>(Sample) / 65535.0f; }
 	/// Converts a sample from a half
 	static inline sample_type convert(const sample_type Sample) { return Sample; }
 	/// Converts a sample from a float
-	static inline sample_type convert(const float Sample) { return Sample; }
+	static inline sample_type convert(const float Sample) { return static_cast<sample_type>(Sample); }
 	/// Converts a sample from a double
-	static inline sample_type convert(const double Sample) { return Sample; }
+	static inline sample_type convert(const double Sample) { return static_cast<sample_type>(static_cast<float>(Sample)); }
 };
 
 /////////////////////////////////////////////////////////////////////////////
@@ -159,13 +193,13 @@ public:
 	static inline sample_type invert(const sample_type& Sample) { return maximum() - Sample; }
 
 	/// Converts a sample from a uint8_t
-	static inline sample_type convert(const boost::uint8_t Sample) { return static_cast<sample_type>(Sample) / 255.0; }
+	static inline sample_type convert(const boost::uint8_t Sample) { return static_cast<sample_type>(Sample) / 255.0f; }
 	/// Converts a sample from a half
 	static inline sample_type convert(const half Sample) { return Sample; }
 	/// Converts a sample from a float
-	static inline sample_type convert(const sample_type Sample) { return Sample; }
+	static inline sample_type convert(const sample_type Sample) { return static_cast<sample_type>(Sample); }
 	/// Converts a sample from a double
-	static inline sample_type convert(const double Sample) { return Sample; }
+	static inline sample_type convert(const double Sample) { return static_cast<sample_type>(Sample); }
 };
 
 /////////////////////////////////////////////////////////////////////////////
@@ -314,60 +348,124 @@ public:
 
 	/// Default constructor sets all samples to zero
 	basic_luma() :
-		luma(sample_traits::minimum()),
-		alpha(sample_traits::opaque())
+		luma(sample_traits::minimum())
 	{
 	}
 
 	/// Straightforward constructor that initializes samples
 	basic_luma(const sample_type Luma) :
-		luma(Luma),
-		alpha(sample_traits::opaque())
-	{
-	}
-
-	basic_luma(const sample_type Luma, const sample_type Alpha) :
-		luma(Luma),
-		alpha(Alpha)
+		luma(Luma)
 	{
 	}
 
 	template<typename ForeignType, typename ForeignTraits>
 	basic_luma(const basic_luma<ForeignType, ForeignTraits>& RHS) :
-		luma(sample_traits::convert(RHS.luma)),
-		alpha(sample_traits::convert(RHS.alpha))
+		luma(sample_traits::convert(RHS.luma))
 	{
 	}
 
 	template<typename ForeignType, typename ForeignTraits>
 	basic_luma(const basic_rgb<ForeignType, ForeignTraits>& RHS) :
-		luma(sample_traits::convert(std::max(RHS.red, std::max(RHS.green, RHS.blue)))),
-		alpha(sample_traits::opaque())
+		luma(sample_traits::convert(std::max(RHS.red, std::max(RHS.green, RHS.blue))))
 	{
 	}
 
 	template<typename ForeignType, typename ForeignTraits>
 	basic_luma(const basic_rgba<ForeignType, ForeignTraits>& RHS) :
-		luma(sample_traits::convert(std::max(RHS.red, std::max(RHS.green, RHS.blue)))),
-		alpha(sample_traits::convert(RHS.alpha))
+		luma(sample_traits::convert(std::max(RHS.red, std::max(RHS.green, RHS.blue))))
 	{
 	}
 
 	basic_luma(const basic_hsv& RHS) :
-		luma(sample_traits::convert(RHS.value)),
-		alpha(sample_traits::opaque())
+		luma(sample_traits::convert(RHS.value))
 	{
 	}
 
 	/// Serialization
 	friend std::ostream& operator << (std::ostream& Stream, const basic_luma<sample_type, sample_traits>& RHS)
 	{
-		Stream << RHS.luma << RHS.alpha;
+		Stream << RHS.luma;
 		return Stream;
 	}
 
 	/// Deserialization
 	friend std::istream& operator >> (std::istream& Stream, basic_luma<sample_type, sample_traits>& RHS)
+	{
+		Stream >> RHS.luma;
+		return Stream;
+	}
+
+	sample_type luma;
+};
+
+/////////////////////////////////////////////////////////////////////////////
+// basic_luma_alpha
+
+/// Encapsulates storage for a luma color sample with alpha
+template<typename SampleType, typename SampleTraits = color_traits<SampleType> >
+class basic_luma_alpha
+{
+public:
+	typedef SampleType sample_type;
+	typedef SampleTraits sample_traits;
+	typedef basic_luma_alpha<sample_type, sample_traits> this_type;
+
+	/// Default constructor sets all samples to zero
+	basic_luma_alpha() :
+		luma(sample_traits::minimum()),
+		alpha(sample_traits::opaque())
+	{
+	}
+
+	/// Straightforward constructor that initializes samples
+	basic_luma_alpha(const sample_type Luma) :
+		luma(Luma),
+		alpha(sample_traits::opaque())
+	{
+	}
+
+	basic_luma_alpha(const sample_type Luma, const sample_type Alpha) :
+		luma(Luma),
+		alpha(Alpha)
+	{
+	}
+
+	template<typename ForeignType, typename ForeignTraits>
+	basic_luma_alpha(const basic_luma_alpha<ForeignType, ForeignTraits>& RHS) :
+		luma(sample_traits::convert(RHS.luma)),
+		alpha(sample_traits::convert(RHS.alpha))
+	{
+	}
+
+	template<typename ForeignType, typename ForeignTraits>
+	basic_luma_alpha(const basic_rgb<ForeignType, ForeignTraits>& RHS) :
+		luma(sample_traits::convert(std::max(RHS.red, std::max(RHS.green, RHS.blue)))),
+		alpha(sample_traits::opaque())
+	{
+	}
+
+	template<typename ForeignType, typename ForeignTraits>
+	basic_luma_alpha(const basic_rgba<ForeignType, ForeignTraits>& RHS) :
+		luma(sample_traits::convert(std::max(RHS.red, std::max(RHS.green, RHS.blue)))),
+		alpha(sample_traits::convert(RHS.alpha))
+	{
+	}
+
+	basic_luma_alpha(const basic_hsv& RHS) :
+		luma(sample_traits::convert(RHS.value)),
+		alpha(sample_traits::opaque())
+	{
+	}
+
+	/// Serialization
+	friend std::ostream& operator << (std::ostream& Stream, const basic_luma_alpha<sample_type, sample_traits>& RHS)
+	{
+		Stream << RHS.luma << RHS.alpha;
+		return Stream;
+	}
+
+	/// Deserialization
+	friend std::istream& operator >> (std::istream& Stream, basic_luma_alpha<sample_type, sample_traits>& RHS)
 	{
 		Stream >> RHS.luma >> RHS.alpha;
 		return Stream;
@@ -498,13 +596,33 @@ public:
 		return !(LHS == RHS);
 	}
 
-	friend std::ostream& operator << (std::ostream& Stream, const basic_rgb<sample_type, sample_traits>& RHS)
+	friend const this_type operator+(const this_type& LHS, const this_type& RHS)
+	{
+		return this_type(LHS.red + RHS.red, LHS.green + RHS.green, LHS.blue + RHS.blue);
+	}
+	
+	friend const this_type operator-(const this_type& LHS, const this_type& RHS)
+	{
+		return this_type(LHS.red - RHS.red, LHS.green - RHS.green, LHS.blue - RHS.blue);
+	}
+	
+	friend const this_type operator*(const this_type& LHS, const double RHS)
+	{
+		return this_type(LHS.red * RHS, LHS.green * RHS, LHS.blue * RHS);
+	}
+
+	friend const this_type operator*(const double LHS, const this_type& RHS)
+	{
+		return RHS * LHS;
+	}
+
+	friend std::ostream& operator<<(std::ostream& Stream, const basic_rgb<sample_type, sample_traits>& RHS)
 	{
 		Stream << RHS.red << " " << RHS.green << " " << RHS.blue;
 		return Stream;
 	}
 
-	friend std::istream& operator >> (std::istream& Stream, basic_rgb<sample_type, sample_traits>& RHS)
+	friend std::istream& operator>>(std::istream& Stream, basic_rgb<sample_type, sample_traits>& RHS)
 	{
 		Stream >> RHS.red >> RHS.green >> RHS.blue;
 		return Stream;
@@ -563,6 +681,15 @@ public:
 	{
 	}
 
+	template<typename ForeignType, typename ForeignTraits>
+	basic_rgba(const basic_luma_alpha<ForeignType, ForeignTraits>& RHS) :
+		red(sample_traits::convert(RHS.luma)),
+		green(sample_traits::convert(RHS.luma)),
+		blue(sample_traits::convert(RHS.luma)),
+		alpha(sample_traits::convert(RHS.alpha))
+	{
+	}
+
 	template<typename ForeignType, typename ForeignTraits>
 	basic_rgba(const basic_rgb<ForeignType, ForeignTraits>& RHS) :
 		red(sample_traits::convert(RHS.red)),
