@@ -25,11 +25,8 @@
 		\author Tim Shead (tshead@k-3d.com)
 */
 
-#ifndef SDPSL_H
-#include <sdpsl/sdpsl.h>
-#endif // !SDPSL_H
-
 #include "algebra.h"
+#include "bounding_box.h"
 #include "color.h"
 #include "iunknown.h"
 #include "vectors.h"
@@ -41,13 +38,11 @@
 #include <list>
 #include <vector>
 
-// Forward declarations
 namespace boost { namespace filesystem { class path; } }
 
 namespace k3d
 {
 
-// Forward declarations
 class iprojection;
 class irender_frame;
 
@@ -103,6 +98,11 @@ typedef boost::uint32_t light_handle;
 /// RenderMan object handle datatype
 typedef boost::uint32_t object_handle;
 
+/// Filesystem path datatype
+typedef boost::filesystem::path path;
+/// 3D bounding-box datatype
+typedef k3d::bounding_box3 bound;
+
 /// RenderMan integer array
 typedef std::vector<integer> integers;
 /// RenderMan unsigned integer array
@@ -134,6 +134,8 @@ typedef enum
 	FACEVARYING
 } storage_class_t;
 
+std::ostream& operator<<(std::ostream& Stream, const storage_class_t RHS);
+
 /// Defines a name-value pair for passing RenderMan parameter lists
 /** \note Although parameter can safely hold any value type, only the
  * integer, real, string, point, vector, normal, color, hpoint and matrix
@@ -260,9 +262,10 @@ public:
 	virtual void RiPointsPolygonsV(const unsigned_integers& VertexCounts, const unsigned_integers& VertexIDs, const parameter_list& Parameters = parameter_list()) = 0;
 	virtual	void RiPointsV(const unsigned_integer VertexCount, const parameter_list& Parameters = parameter_list()) = 0;
 	virtual void RiPolygonV(const unsigned_integer VertexCount, const parameter_list& Parameters = parameter_list()) = 0;
+	virtual void RiProcDelayedReadArchive(const path& Archive, const bound& BoundingBox) = 0;
 	virtual void RiProjectionV(const string& Name, const parameter_list& Parameters = parameter_list()) = 0;
 	virtual void RiQuantize(const string& Type, const integer One, const integer QMin, const integer QMax, const real Amplitude) = 0;
-	virtual void RiReadArchive(const string& Archive) = 0;
+	virtual void RiReadArchive(const path& Archive) = 0;
 	virtual void RiRelativeDetail(const real RelativeDetail) = 0;
 	virtual void RiReverseOrientation() = 0;
 	virtual void RiRotate(const real angle, const real DX, const real DY, const real DZ) = 0;
@@ -289,6 +292,14 @@ public:
 	virtual void RiWorldBegin() = 0;
 	virtual void RiWorldEnd() = 0;
 
+	// New Aqsis-specific calls introduced to support layered shaders
+	//
+	/** \todo Come up with a better way to handle non-standard calls */
+	virtual void RiShaderLayerV(const std::string& type, const std::string& name, const std::string& layername, const parameter_list& Parameters = parameter_list()) = 0;
+	/** \todo Come up with a better way to handle non-standard calls */
+	virtual void RiConnectShaderLayers(const std::string& type, const std::string& layer1, const std::string& variable1, const std::string& layer2, const std::string& variable2) = 0;
+
+	
 protected:
 	irender_engine() {}
 	irender_engine(const irender_engine& ) {}
@@ -352,7 +363,6 @@ class irenderable :
 	public virtual iunknown
 {
 public:
-	virtual void renderman_pre_render(const render_state& State) = 0;
 	virtual void renderman_render(const render_state& State) = 0;
 
 protected:
