@@ -29,10 +29,10 @@ namespace k3d
 
 // Forward declarations
 class idocument;
-class iobject;
+class inode;
 
 /// Returns the default material for application to new objects (could return NULL!)
-iobject* default_material(k3d::idocument& Document);
+inode* default_material(k3d::idocument& Document);
 
 } // namespace k3d
 
