@@ -24,6 +24,7 @@
 
 #include "expression_grammar.h"
 #include "measurement.h"
+#include "result.h"
 
 #include <stdexcept>
 
@@ -139,7 +140,7 @@ void format(std::ostream& Stream, const double Value, const std::type_info& Unit
 			else if(Units == typeid(scalar))
 				Stream << Value;
 			else
-				std::cerr << __PRETTY_FUNCTION__ << ": unknown physical unit" << std::endl;
+				log() << error << "Unknown physical unit" << std::endl;
 }
 
 const angle_units_t& angle_units()
@@ -433,4 +434,3 @@ const volume& volume_units(const std::string Symbol)
 
 } // namespace k3d
 
-
