@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 1995-2004, Timothy M. Shead
+// Copyright (c) 1995-2005, Timothy M. Shead
 //
 // Contact: tshead@k-3d.com
 //
@@ -29,6 +29,8 @@ namespace k3d
 /////////////////////////////////////////////////////////////////////////////
 // point
 
+k3d_instantiate_memory_cache(point);
+
 point::point(const vector3& Position) :
 	position(Position)
 {
@@ -42,6 +44,8 @@ point::point(const double X, const double Y, const double Z) :
 /////////////////////////////////////////////////////////////////////////////
 // point_group
 
+k3d_instantiate_memory_cache(point_group);
+
 point_group::point_group() :
 	material(0)
 {
@@ -50,14 +54,54 @@ point_group::point_group() :
 /////////////////////////////////////////////////////////////////////////////
 // split_edge
 
+k3d_instantiate_memory_cache(split_edge);
+
+split_edge::~split_edge()
+{
+	// Acknowledge companion he's now alone
+	if(companion)
+		companion->companion = 0;
+}
+
 /////////////////////////////////////////////////////////////////////////////
 // face
 
-face::face(split_edge* FirstEdge) :
-	first_edge(FirstEdge)
+k3d_instantiate_memory_cache(face);
+
+face::face(split_edge* FirstEdge, imaterial* Material) :
+	first_edge(FirstEdge),
+	material(Material)
 {
 }
 
+face::~face()
+{
+	for(k3d::split_edge* edge = first_edge; edge; )
+	{
+		k3d::split_edge* next_edge = edge->face_clockwise;
+
+		delete edge;
+		if(next_edge == first_edge)
+			break;
+
+		edge = next_edge;
+	}
+
+	for(holes_t::iterator hole = holes.begin(); hole != holes.end(); ++hole)
+	{
+		for(k3d::split_edge* edge = *hole; edge; )
+		{
+			k3d::split_edge* next_edge = edge->face_clockwise;
+
+			delete edge;
+			if(next_edge == *hole)
+				break;
+
+			edge = next_edge;
+		}
+	}
+}
+
 /////////////////////////////////////////////////////////////////////////////
 // normal
 
@@ -90,16 +134,16 @@ vector3 normal(const face& Face)
 /////////////////////////////////////////////////////////////////////////////
 // polyhedron
 
+k3d_instantiate_memory_cache(polyhedron);
+
 polyhedron::polyhedron() :
-	type(POLYGONS),
-	material(0)
+	type(POLYGONS)
 {
 }
 
 polyhedron::~polyhedron()
 {
 	std::for_each(faces.begin(), faces.end(), delete_object());
-	std::for_each(edges.begin(), edges.end(), delete_object());
 }
 
 std::ostream& operator<<(std::ostream& Stream, const polyhedron::type_t& RHS)
@@ -127,7 +171,7 @@ std::istream& operator>>(std::istream& Stream, polyhedron::type_t& RHS)
 	else if(buffer == "catmull_clark")
 		RHS = polyhedron::CATMULL_CLARK_SUBDIVISION_MESH;
 	else
-		std::cerr << error << "Unknown polyhedron type [" << buffer << "]" << std::endl;
+		log() << error << "Unknown polyhedron type [" << buffer << "]" << std::endl;
 
 	return Stream;
 }
@@ -202,7 +246,7 @@ void set_companions(k3d::polyhedron& Polyhedron)
 						{
 							k3d::split_edge* other_edge = edge->second;
 							if(!other_edge)
-								std::cerr << debug << "An edge is shared by more than two faces" << std::endl;
+								log() << debug << "An edge is shared by more than two faces" << std::endl;
 							else
 								{
 									k3d::join_edges(*current_edge, *(edge->second));
@@ -223,9 +267,13 @@ void set_companions(k3d::polyhedron& Polyhedron)
 /////////////////////////////////////////////////////////////////////////////
 // linear_curve
 
+k3d_instantiate_memory_cache(linear_curve);
+
 /////////////////////////////////////////////////////////////////////////////
 // linear_curve_group
 
+k3d_instantiate_memory_cache(linear_curve_group);
+
 linear_curve_group::linear_curve_group() :
 	wrap(false),
 	material(0)
@@ -240,9 +288,13 @@ linear_curve_group::~linear_curve_group()
 /////////////////////////////////////////////////////////////////////////////
 // cubic_curve
 
+k3d_instantiate_memory_cache(cubic_curve);
+
 /////////////////////////////////////////////////////////////////////////////
 // cubic_curve_group
 
+k3d_instantiate_memory_cache(cubic_curve_group);
+
 cubic_curve_group::cubic_curve_group() :
 	wrap(false),
 	material(0)
@@ -257,6 +309,8 @@ cubic_curve_group::~cubic_curve_group()
 /////////////////////////////////////////////////////////////////////////////
 // nucurve
 
+k3d_instantiate_memory_cache(nucurve);
+
 nucurve::nucurve() :
 	order(2)
 {
@@ -265,6 +319,8 @@ nucurve::nucurve() :
 /////////////////////////////////////////////////////////////////////////////
 // nucurve_group
 
+k3d_instantiate_memory_cache(nucurve_group);
+
 nucurve_group::nucurve_group() :
 	material(0)
 {
@@ -278,6 +334,8 @@ nucurve_group::~nucurve_group()
 /////////////////////////////////////////////////////////////////////////////
 // bilinear_patch
 
+k3d_instantiate_memory_cache(bilinear_patch);
+
 bilinear_patch::bilinear_patch() :
 	material(0)
 {
@@ -286,6 +344,8 @@ bilinear_patch::bilinear_patch() :
 /////////////////////////////////////////////////////////////////////////////
 // bicubic_patch
 
+k3d_instantiate_memory_cache(bicubic_patch);
+
 bicubic_patch::bicubic_patch() :
 	material(0)
 {
@@ -294,6 +354,8 @@ bicubic_patch::bicubic_patch() :
 /////////////////////////////////////////////////////////////////////////////
 // nupatch
 
+k3d_instantiate_memory_cache(nupatch);
+
 nupatch::nupatch() :
 	u_order(2),
 	v_order(2),
@@ -304,6 +366,8 @@ nupatch::nupatch() :
 /////////////////////////////////////////////////////////////////////////////
 // blobby
 
+k3d_instantiate_memory_cache(blobby);
+
 blobby::blobby(opcode* Opcode) :
 	root(Opcode),
 	material(0)
@@ -489,7 +553,7 @@ void blobby::multiply::accept(visitor& Visitor)
 
 blobby::opcode* blobby::max::clone()
 {
-	max* result = new max(*this);
+	blobby::max* result = new blobby::max(*this);
 	result->clone_operands();
 	return result;
 }
@@ -504,7 +568,7 @@ void blobby::max::accept(visitor& Visitor)
 
 blobby::opcode* blobby::min::clone()
 {
-	min* result = new min(*this);
+	blobby::min* result = new blobby::min(*this);
 	result->clone_operands();
 	return result;
 }
@@ -517,6 +581,8 @@ void blobby::min::accept(visitor& Visitor)
 /////////////////////////////////////////////////////////////////////////////
 // mesh
 
+k3d_instantiate_memory_cache(mesh);
+
 mesh::mesh()
 {
 }
@@ -532,10 +598,28 @@ mesh::~mesh()
 	std::for_each(points.begin(), points.end(), delete_object());
 }
 
+/////////////////////////////////////////////////////////////////////////////
+// triangulate
+
+#include "mesh_triangulate_detail.h"
+
+template<typename InputFaceIterator, typename OutputFaceIterator, typename OutputPointIterator>
+void triangulate(InputFaceIterator Begin, InputFaceIterator End, OutputFaceIterator OutputFaces, OutputPointIterator OutputPoints)
+{
+	detail::glu_triangulator_t<OutputFaceIterator, OutputPointIterator> triangulator(OutputFaces, OutputPoints);
+	for(InputFaceIterator face = Begin; face != End; ++face)
+		triangulator(**face);
+}
+
+void triangulate(const polyhedron::faces_t& Faces, polyhedron::faces_t& NewFaces, mesh::points_t& NewPoints)
+{
+	triangulate(Faces.begin(), Faces.end(), std::back_inserter(NewFaces), std::back_inserter(NewPoints));
+}
+
 /////////////////////////////////////////////////////////////////////////////
 // add_unit_cube
 
-void add_unit_cube(mesh& Mesh, polyhedron& Polyhedron)
+void add_unit_cube(mesh& Mesh, polyhedron& Polyhedron, imaterial* const Material)
 {
 	// Create points ...
 	boost::multi_array<point*, 3> points(boost::extents[2][2][2]);
@@ -619,19 +703,15 @@ void add_unit_cube(mesh& Mesh, polyhedron& Polyhedron)
 		for(unsigned long j = 0; j != 4; ++j)
 			edges[i][j]->face_clockwise = edges[i][(j+1)%4];
 
-	for(unsigned long i = 0; i != 6; ++i)
-		for(unsigned long j = 0; j != 4; ++j)
-			Polyhedron.edges.push_back(edges[i][j]);
-
 	// Create faces ...
 	for(unsigned long i = 0; i != 6; ++i)
-		Polyhedron.faces.push_back(new face(edges[i][0]));
+		Polyhedron.faces.push_back(new face(edges[i][0], Material));
 }
 
 /////////////////////////////////////////////////////////////////////////////
 // add_grid
 
-grid_results_t add_grid(mesh& Mesh, polyhedron& Polyhedron, const unsigned long Rows, const unsigned long Columns, const bool StitchTop, const bool StitchSide)
+grid_results_t add_grid(mesh& Mesh, polyhedron& Polyhedron, const unsigned long Rows, const unsigned long Columns, const bool StitchTop, const bool StitchSide, imaterial* const Material)
 {
 	// Sanity checks ...
 	assert(Rows);
@@ -669,9 +749,6 @@ grid_results_t add_grid(mesh& Mesh, polyhedron& Polyhedron, const unsigned long
 
 					for(unsigned long i = 0; i != 4; ++i)
 						edges[row][column][i]->face_clockwise = edges[row][column][(i+1)%4];
-
-					for(unsigned long i = 0; i != 4; ++i)
-						Polyhedron.edges.push_back(edges[row][column][i]);
 				}
 		}
 
@@ -697,7 +774,7 @@ grid_results_t add_grid(mesh& Mesh, polyhedron& Polyhedron, const unsigned long
 		{
 			for(unsigned long column = 0; column != face_columns; ++column)
 				{
-					face* const new_face = new face(edges[row][column][0]);
+					face* const new_face = new face(edges[row][column][0], Material);
 					Polyhedron.faces.push_back(new_face);
 				}
 		}
@@ -798,43 +875,54 @@ void deep_copy(const mesh& Input, mesh& Output)
 	for(mesh::polyhedra_t::const_iterator pn = Input.polyhedra.begin(); pn != Input.polyhedra.end(); ++pn)
 		{
 			polyhedron* const new_polyhedron = new polyhedron(**pn);
+			Output.polyhedra.push_back(new_polyhedron);
 
 			// Duplicate edges ...
 			typedef std::map<split_edge*, split_edge*> edge_map_t;
 			edge_map_t edge_map;
 			edge_map[0] = 0;
-			for(polyhedron::edges_t::iterator edge = new_polyhedron->edges.begin(); edge != new_polyhedron->edges.end(); ++edge)
+			for(polyhedron::faces_t::iterator f = new_polyhedron->faces.begin(); f != new_polyhedron->faces.end(); ++f)
 				{
-					split_edge* const new_edge = new split_edge(**edge);
-					edge_map.insert(std::make_pair(*edge, new_edge));
-					*edge = new_edge;
+					for(split_edge* edge = (*f)->first_edge; edge; edge = edge->face_clockwise)
+						{
+							edge_map.insert(std::make_pair(edge, new split_edge(*edge)));
+							if(edge->face_clockwise == (*f)->first_edge)
+								break;
+						}
+
+					for(face::holes_t::iterator hole = (*f)->holes.begin(); hole != (*f)->holes.end(); ++hole)
+						{
+							for(split_edge* edge = *hole; edge; edge = edge->face_clockwise)
+								{
+									edge_map.insert(std::make_pair(edge, new split_edge(*edge)));
+									if(edge->face_clockwise == (*hole))
+										break;
+								}
+						}
 				}
 
-			// Re-link edges ...
+			// Relink edges ...
 			for(edge_map_t::iterator edge = edge_map.begin(); edge != edge_map.end(); ++edge)
 				{
 					if(!edge->first)
 						continue;
 
 					edge->second->vertex = point_map[edge->second->vertex];
-					edge->second->face_clockwise = edge_map.find(edge->second->face_clockwise)->second;
-					edge->second->companion = edge_map.find(edge->second->companion)->second;
+					edge->second->face_clockwise = edge_map[edge->second->face_clockwise];
+					edge->second->companion = edge_map[edge->second->companion];
 				}
 
-			// Duplicate faces ...
+			// Duplicate faces and re-link edges ...
 			for(polyhedron::faces_t::iterator f = new_polyhedron->faces.begin(); f != new_polyhedron->faces.end(); ++f)
 				{
 					face* const new_face = new face(**f);
-					new_face->first_edge = edge_map[new_face->first_edge];
 
-					// Duplicate holes ...
+					new_face->first_edge = edge_map[new_face->first_edge];
 					for(face::holes_t::iterator hole = new_face->holes.begin(); hole != new_face->holes.end(); ++hole)
-						*hole = edge_map.find(*hole)->second;
+						*hole = edge_map[*hole];
 
 					*f = new_face;
 				}
-
-			Output.polyhedra.push_back(new_polyhedron);
 		}
 
 	// Duplicate linear curve groups ...
@@ -918,6 +1006,40 @@ void deep_copy(const mesh& Input, mesh& Output)
 		}
 }
 
+/////////////////////////////////////////////////////////////////////////////
+// segment_count
+
+const unsigned long segment_count(const linear_curve& Curve, const bool Wrap)
+{
+	if(Wrap)
+		return Curve.control_points.size();
+
+	return Curve.control_points.size() ? Curve.control_points.size() - 1 : 0;
+}
+
+const unsigned long segment_count(const cubic_curve& Curve, const bool Wrap)
+{
+	const unsigned long vstep = 3;
+
+	if(Wrap)
+		return Curve.control_points.size() / vstep;
+
+	return Curve.control_points.size() > 3 ? ((Curve.control_points.size() - 4) / vstep) + 1 : 0;
+}
+
+/////////////////////////////////////////////////////////////////////////////
+// varying count
+
+const unsigned long varying_count(const linear_curve& Curve, const bool Wrap)
+{
+	return Wrap ? segment_count(Curve, Wrap) : segment_count(Curve, Wrap) + 1;
+}
+
+const unsigned long varying_count(const cubic_curve& Curve, const bool Wrap)
+{
+	return Wrap ? segment_count(Curve, Wrap) : segment_count(Curve, Wrap) + 1;
+}
+
 /////////////////////////////////////////////////////////////////////////////
 // is_valid
 
@@ -928,28 +1050,49 @@ bool is_valid(const polyhedron& Polyhedron)
 		{
 			// This is obviously wrong!!!
 			return_val_if_fail(*face, false);
-		}
 
-	// For every edge ...
-	for(polyhedron::edges_t::const_iterator edge = Polyhedron.edges.begin(); edge != Polyhedron.edges.end(); ++edge)
-		{
-			// This is obviously wrong!!!
-			return_val_if_fail(*edge, false);
+			// Check face edges
+			split_edge* first_edge = (*face)->first_edge;
 
-			// Every edge should have a vertex ...
-			return_val_if_fail((*edge)->vertex, false);
-
-			// Every edge should have a neighbor ...
-			return_val_if_fail((*edge)->face_clockwise, false);
-
-			// For edges with companions ...
-			if((*edge)->companion)
+			split_edge* edge = first_edge;
+			unsigned long edge_count = 0;
+			do
 				{
-					// Companions had better point to each other ...
-					return_val_if_fail((*edge)->companion->companion == (*edge), false);
+					// Every edge should have a neighbour ...
+					return_val_if_fail(edge, false);
 
-					// Companions had better NOT share the same vertex ...
-					return_val_if_fail((*edge)->vertex != (*edge)->companion->vertex, false);
+					// Every edge should have a vertex ...
+					return_val_if_fail(edge->vertex, false);
+
+					// For edges with companions ...
+					if(edge->companion)
+						{
+							// Companions had better point to each other ...
+							return_val_if_fail(edge->companion->companion == edge, false);
+
+							// Companions had better NOT share the same vertex ...
+							return_val_if_fail(edge->vertex != edge->companion->vertex, false);
+						}
+
+					++edge_count;
+					edge = edge->face_clockwise;
+				}
+			while(edge != first_edge);
+
+			// Every face should have at least three edges ...
+			return_val_if_fail(edge_count > 2, false);
+
+			// Check holes
+			for(k3d::face::holes_t::iterator hole = (*face)->holes.begin(); hole != (*face)->holes.end(); ++hole)
+				{
+					for(k3d::split_edge* edge = *hole; edge; edge = edge->face_clockwise)
+						{
+							// Every edge should have a vertex ...
+							return_val_if_fail(edge->vertex, false);
+
+							if(edge->face_clockwise == (*hole))
+								break;
+						}
 				}
 		}
 
@@ -1007,13 +1150,19 @@ bool is_solid(const polyhedron& Polyhedron)
 	if(!is_valid(Polyhedron))
 		return false;
 
-	if(Polyhedron.edges.empty())
-		return false;
-
-	for(polyhedron::edges_t::const_iterator edge = Polyhedron.edges.begin(); edge != Polyhedron.edges.end(); ++edge)
+	for(polyhedron::faces_t::const_iterator face = Polyhedron.faces.begin(); face != Polyhedron.faces.end(); ++face)
 		{
-			if(0 == (**edge).companion)
-				return false;
+			split_edge* first_edge = (*face)->first_edge;
+			split_edge* edge = first_edge;
+
+			do
+				{
+					// There should be no border edge
+					return_val_if_fail(edge->companion, false);
+
+					edge = edge->face_clockwise;
+				}
+			while(edge != first_edge);
 		}
 
 	return true;
@@ -1022,9 +1171,9 @@ bool is_solid(const polyhedron& Polyhedron)
 /////////////////////////////////////////////////////////////////////////////
 // bounds
 
-const bounding_box bounds(const mesh& Mesh)
+const bounding_box3 bounds(const mesh& Mesh)
 {
-	bounding_box results;
+	bounding_box3 results;
 	for(mesh::points_t::const_iterator point = Mesh.points.begin(); point != Mesh.points.end(); ++point)
 		results.insert((*point)->position);
 
