@@ -2,7 +2,7 @@
 #define K3DSDK_MESH_H
 
 // K-3D
-// Copyright (c) 1995-2004, Timothy M. Shead
+// Copyright (c) 1995-2005, Timothy M. Shead
 //
 // Contact: tshead@k-3d.com
 //
@@ -22,7 +22,8 @@
 
 #include "algebra.h"
 #include "bounding_box.h"
-#include "result.h"
+#include "gl.h"
+#include "memory.h"
 #include "selectable.h"
 #include "utility.h"
 
@@ -35,8 +36,6 @@
 #include <string>
 #include <vector>
 
-#include <sdpgl/sdpgl.h>
-
 namespace k3d
 {
 
@@ -54,7 +53,8 @@ typedef std::map<std::string, boost::any> parameters_t;
 
 /// Encapsulates a point in 3D space
 class point :
-	public selectable
+	public selectable,
+	public memory::cache<point>
 {
 public:
 	point(const vector3& Position);
@@ -73,7 +73,8 @@ public:
 
 /// Encapsulates a group of rendered points
 class point_group :
-	public selectable
+	public selectable,
+	public memory::cache<point_group>
 {
 public:
 	point_group();
@@ -92,7 +93,8 @@ public:
 
 /// Encapsulates a split-edge data structure for representing topology information in a polygon mesh
 class split_edge :
-	public selectable
+	public selectable,
+	public memory::cache<split_edge>
 {
 public:
 	split_edge(point* Vertex, split_edge* FaceClockwise = 0, split_edge* Companion = 0) :
@@ -101,6 +103,7 @@ public:
 		companion(Companion)
 	{
 	}
+	~split_edge();
 
 	/// Stores a reference to the edge vertex
 	point* vertex;
@@ -128,22 +131,39 @@ inline void loop_edges(IteratorType Begin, IteratorType End)
 	if(Begin == End)
 		return;
 
-	IteratorType i;
-	for(i = Begin; i+1 != End; ++i)
-		(*i)->face_clockwise = *(i+1);
+	IteratorType i = Begin;
+	IteratorType j = i;
+	while(++j != End)
+	{
+		(*i)->face_clockwise = *j;
+
+		j = ++i;
+	}
 
 	(*i)->face_clockwise = *Begin;
 }
 
+/// Convenience function that returns face anticlockwise edge
+inline split_edge* face_anticlockwise(split_edge* Edge)
+{
+	split_edge* current_edge = Edge;
+	while(current_edge && current_edge->face_clockwise != Edge)
+		current_edge = current_edge->face_clockwise;
+
+	return current_edge;
+}
+
 /////////////////////////////////////////////////////////////////////////////
 // face
 
 /// Encapsulates a polygonal face
 class face :
-	public selectable
+	public selectable,
+	public memory::cache<face>
 {
 public:
-	face(split_edge* FirstEdge);
+	face(split_edge* FirstEdge, imaterial* Material);
+	~face();
 
 	/// Points to the first edge in the loop of edges that define the polygon
 	split_edge* first_edge;
@@ -151,23 +171,14 @@ public:
 	typedef std::vector<split_edge*> holes_t;
 	/// Contains any holes in the polygon face
 	holes_t holes;
+	/// Stores a reference to the (optional) material for the face
+	imaterial* material;
 	/// Stores uniform data for the face
 	parameters_t uniform_data;
 	/// Stores tag (SDS) data for the face
 	parameters_t tags;
 };
 
-#include "mesh_triangulate_detail.h"
-
-/// Decomposes a collection of faces into triangles
-template<typename InputFaceIterator, typename OutputFaceIterator, typename OutputEdgeIterator, typename OutputPointIterator>
-void triangulate(InputFaceIterator Begin, InputFaceIterator End, OutputFaceIterator OutputFaces, OutputEdgeIterator OutputEdges, OutputPointIterator OutputPoints)
-{
-	detail::glu_triangulator_t<OutputFaceIterator, OutputEdgeIterator, OutputPointIterator> triangulator(OutputFaces, OutputEdges, OutputPoints);
-	for(InputFaceIterator face = Begin; face != End; ++face)
-		triangulator(**face);
-}
-
 /// Calculates the normal for an edge loop (returns a zero-length normal for degenerate cases)
 vector3 normal(const split_edge* const Loop);
 /// Calculates the normal for a face (returns a zero-length normal for degenerate cases)
@@ -178,15 +189,13 @@ vector3 normal(const face& Face);
 
 /// Encapsulates a manifold polyhedron composed of polygon faces
 class polyhedron :
-	public selectable
+	public selectable,
+	public memory::cache<polyhedron>
 {
 public:
 	polyhedron();
 	~polyhedron();
 
-	typedef std::vector<split_edge*> edges_t;
-	edges_t edges;
-
 	typedef std::vector<face*> faces_t;
 	faces_t faces;
 
@@ -198,8 +207,6 @@ public:
 
 	/// Stores the polyhedron type
 	type_t type;
-	/// Stores a reference to the (optional) material for the polyhedron
-	imaterial* material;
 	/// Stores constant data for the polyhedron
 	parameters_t constant_data;
 	/// Stores tag (SDS) data for the polyhedron
@@ -210,14 +217,15 @@ public:
 };
 
 /// Convenience function for setting all companions of a raw polygon edges
-void set_companions(k3d::polyhedron& Polyhedron);
+void set_companions(polyhedron& Polyhedron);
 
 /////////////////////////////////////////////////////////////////////////////
 // linear_curve
 
 /// Encapsulates a linear curve
 class linear_curve :
-	public selectable
+	public selectable,
+	public memory::cache<linear_curve>
 {
 public:
 	/// Defines storage for the set of curve control points
@@ -237,7 +245,8 @@ public:
 
 /// Encapsulates a collection of linear curves
 class linear_curve_group :
-	public selectable
+	public selectable,
+	public memory::cache<linear_curve_group>
 {
 public:
 	linear_curve_group();
@@ -260,7 +269,8 @@ public:
 
 /// Encapsulates a cubic curve
 class cubic_curve :
-	public selectable
+	public selectable,
+	public memory::cache<cubic_curve>
 {
 public:
 	/// Defines storage for the set of curve control points
@@ -280,7 +290,8 @@ public:
 
 /// Encapsulates a collection of cubic curves
 class cubic_curve_group :
-	public selectable
+	public selectable,
+	public memory::cache<cubic_curve_group>
 {
 public:
 	cubic_curve_group();
@@ -303,7 +314,8 @@ public:
 
 /// Encapsulates a NURBS curve - note: there is no equivalent RenderMan primitive, so these can't be rendered - take a look at cubic_curve, instead.
 class nucurve :
-	public selectable
+	public selectable,
+	public memory::cache<nucurve>
 {
 public:
 	nucurve();
@@ -331,7 +343,8 @@ public:
 
 /// Encapsulates a collection of NURBS curves - note: there is no equivalent RenderMan primitive, so these can't be rendered - take a look at cubic_curve_group, instead
 class nucurve_group :
-	public selectable
+	public selectable,
+	public memory::cache<nucurve_group>
 {
 public:
 	nucurve_group();
@@ -350,7 +363,8 @@ public:
 
 /// Encapsulates a bilinear patch
 class bilinear_patch :
-	public selectable
+	public selectable,
+	public memory::cache<bilinear_patch>
 {
 public:
 	bilinear_patch();
@@ -375,7 +389,8 @@ public:
 
 /// Encapsulates a bicubic patch
 class bicubic_patch :
-	public selectable
+	public selectable,
+	public memory::cache<bicubic_patch>
 {
 public:
 	bicubic_patch();
@@ -399,7 +414,8 @@ public:
 
 /// Encapsulates a NURBS patch
 class nupatch :
-	public selectable
+	public selectable,
+	public memory::cache<nupatch>
 {
 public:
 	nupatch();
@@ -436,7 +452,8 @@ public:
 // blobby
 
 /// Encapsulates a RenderMan blobby (implicit surface) primitive as a tree
-class blobby
+class blobby :
+	public memory::cache<blobby>
 {
 public:
 	// Forward declarations
@@ -616,6 +633,7 @@ public:
 
 	protected:
 		visitor() {}
+		virtual ~visitor() {}
 		visitor(const visitor&) {}
 		visitor& operator=(const visitor&) { return *this; }
 	};
@@ -626,7 +644,8 @@ public:
 
 /// Encapsulates a collection of geometry
 class mesh :
-	public selectable
+	public selectable,
+	public memory::cache<mesh>
 {
 public:
 	mesh();
@@ -687,15 +706,150 @@ private:
 	mesh& operator=(const mesh& RHS);
 };
 
+/// Function template in the spirit of std::for_each that applies a functor to every point in a mesh
+template<typename T>
+void for_each_point(mesh& Mesh, T Functor)
+{
+	for(mesh::points_t::iterator point = Mesh.points.begin(); point != Mesh.points.end(); ++point)
+		Functor(**point);
+}
+
+/// Function template in the spirit of std::for_each that applies a functor to every edge in a mesh
+template<typename T>
+void for_each_edge(mesh& Mesh, T Functor)
+{
+	for(mesh::polyhedra_t::iterator polyhedron = Mesh.polyhedra.begin(); polyhedron != Mesh.polyhedra.end(); ++polyhedron)
+	{
+		for(polyhedron::faces_t::const_iterator face = (*polyhedron)->faces.begin(); face != (*polyhedron)->faces.end(); ++face)
+		{
+			for(split_edge* edge = (*face)->first_edge; edge; edge = edge->face_clockwise)
+			{
+				Functor(*edge);
+
+				if(edge->face_clockwise == (*face)->first_edge)
+					break;
+			}
+
+			// Face holes
+			for(face::holes_t::iterator hole = (*face)->holes.begin(); hole != (*face)->holes.end(); ++hole)
+			{
+				for(split_edge* edge = *hole; edge; edge = edge->face_clockwise)
+				{
+					Functor(*edge);
+
+					if(edge->face_clockwise == (*hole))
+						break;
+				}
+			}
+		}
+	}
+}
+
+/// Function template in the spirit of std::for_each that applies a functor to every face in a mesh
+template<typename T>
+void for_each_face(mesh& Mesh, T Functor)
+{
+	for(mesh::polyhedra_t::iterator polyhedron = Mesh.polyhedra.begin(); polyhedron != Mesh.polyhedra.end(); ++polyhedron)
+	{
+		for(polyhedron::faces_t::const_iterator face = (*polyhedron)->faces.begin(); face != (*polyhedron)->faces.end(); ++face)
+			Functor(**face);
+	}
+}
+
+/// Function template in the spirit of std::for_each that applies a functor to every part of a mesh
+template<typename T>
+T for_each_component(mesh& Mesh, T Functor)
+{
+	Functor(Mesh);
+
+	for(mesh::points_t::iterator point = Mesh.points.begin(); point != Mesh.points.end(); ++point)
+		Functor(**point);
+
+	for(mesh::polyhedra_t::iterator polyhedron = Mesh.polyhedra.begin(); polyhedron != Mesh.polyhedra.end(); ++polyhedron)
+	{
+		Functor(**polyhedron);
+
+		for(polyhedron::faces_t::iterator face = (*polyhedron)->faces.begin(); face != (*polyhedron)->faces.end(); ++face)
+		{
+			Functor(**face);
+
+			for(split_edge* edge = (*face)->first_edge; edge; edge = edge->face_clockwise)
+			{
+				Functor(*edge);
+
+				if(edge->face_clockwise == (*face)->first_edge)
+					break;
+			}
+
+			// Face holes
+			for(face::holes_t::iterator hole = (*face)->holes.begin(); hole != (*face)->holes.end(); ++hole)
+			{
+				for(split_edge* edge = *hole; edge; edge = edge->face_clockwise)
+				{
+					Functor(*edge);
+
+					if(edge->face_clockwise == (*hole))
+						break;
+				}
+			}
+		}
+	}
+
+	for(mesh::linear_curve_groups_t::iterator group = Mesh.linear_curve_groups.begin(); group != Mesh.linear_curve_groups.end(); ++group)
+	{
+		Functor(**group);
+		for(linear_curve_group::curves_t::iterator curve = (*group)->curves.begin(); curve != (*group)->curves.end(); ++curve)
+			Functor(**curve);
+	}
+
+	for(mesh::cubic_curve_groups_t::iterator group = Mesh.cubic_curve_groups.begin(); group != Mesh.cubic_curve_groups.end(); ++group)
+	{
+		Functor(**group);
+		for(cubic_curve_group::curves_t::iterator curve = (*group)->curves.begin(); curve != (*group)->curves.end(); ++curve)
+			Functor(**curve);
+	}
+
+	for(mesh::nucurve_groups_t::iterator group = Mesh.nucurve_groups.begin(); group != Mesh.nucurve_groups.end(); ++group)
+	{
+		Functor(**group);
+		for(nucurve_group::curves_t::iterator curve = (*group)->curves.begin(); curve != (*group)->curves.end(); ++curve)
+			Functor(**curve);
+	}
+
+	for(mesh::bilinear_patches_t::iterator patch = Mesh.bilinear_patches.begin(); patch != Mesh.bilinear_patches.end(); ++patch)
+		Functor(**patch);
+
+	for(mesh::bicubic_patches_t::iterator patch = Mesh.bicubic_patches.begin(); patch != Mesh.bicubic_patches.end(); ++patch)
+		Functor(**patch);
+
+	for(mesh::nupatches_t::iterator patch = Mesh.nupatches.begin(); patch != Mesh.nupatches.end(); ++patch)
+		Functor(**patch);
+
+	return Functor;
+}
+
+/// Decomposes a collection of faces into triangles
+void triangulate(const polyhedron::faces_t& Faces, polyhedron::faces_t& NewFaces, mesh::points_t& NewPoints);
+
 /// Creates data for a unit cube and appends it to the given Mesh & Polyhedron
-void add_unit_cube(mesh& Mesh, polyhedron& Polyhedron);
+void add_unit_cube(mesh& Mesh, polyhedron& Polyhedron, imaterial* const Material);
 /// Defines a reference to polygonal grid data that maintains the topology in a convenient form
 typedef boost::tuple<boost::multi_array<point*, 2>, boost::multi_array<split_edge*, 3>, boost::multi_array<face*, 2> > grid_results_t;
 /// Creates data for a grid, optionally stitched along either or both axes (e.g. a cylinder or a torus)
-grid_results_t add_grid(mesh& Mesh, polyhedron& Polyhedron, const unsigned long Rows, const unsigned long Columns, const bool StitchTop, const bool StitchSide);
+grid_results_t add_grid(mesh& Mesh, polyhedron& Polyhedron, const unsigned long Rows, const unsigned long Columns, const bool StitchTop, const bool StitchSide, imaterial* const Material);
 /// Copies the input mesh into the output mesh
 void deep_copy(const mesh& Input, mesh& Output);
 
+/// Returns the number of segments in a linear curve, based on the control point count and wrap state
+const unsigned long segment_count(const linear_curve& Curve, const bool Wrap);
+/// Returns the number of segments in a cubic curve, based on the control point count, basis, and wrap state
+const unsigned long segment_count(const cubic_curve& Curve, const bool Wrap);
+
+/// Returns the number of varying parameters in a linear curve, based on the control point count and wrap state
+const unsigned long varying_count(const linear_curve& Curve, const bool Wrap);
+/// Returns the number of varying parameters in a cubic curve, based on the control point count, basis, and wrap state
+const unsigned long varying_count(const cubic_curve& Curve, const bool Wrap);
+
 /// Returns true iff the given polyhedron contains valid data
 bool is_valid(const polyhedron& Polyhedron);
 /// Returns true iff the given NURBS curve contains valid data
@@ -707,10 +861,9 @@ bool is_valid(const nupatch& Patch);
 bool is_solid(const polyhedron& Polyhedron);
 
 /// Returns a bounding-box containing every point in the given mesh
-const bounding_box bounds(const mesh& Mesh);
+const bounding_box3 bounds(const mesh& Mesh);
 
 } // namespace k3d
 
 #endif // K3DSDK_MESH_H
 
-
