@@ -2,7 +2,7 @@
 #define K3DSDK_SELECTABLE_H
 
 // K-3D
-// Copyright (c) 1995-2004, Timothy M. Shead
+// Copyright (c) 1995-2005, Timothy M. Shead
 //
 // Contact: tshead@k-3d.com
 //
@@ -31,28 +31,37 @@ class selectable :
 {
 public:
 	selectable() :
-		selected(false)
+		visible_selection(false),
+		selection_weight(1.0)
 	{
 	}
-	
-	bool is_selected()
+
+	bool get_visible_selection()
 	{
-		return selected;
+		return visible_selection;
 	}
-	
-	virtual void select()
+
+	void set_visible_selection(const bool Visible)
 	{
-		selected = true;
+		visible_selection = Visible;
 	}
-	
-	virtual void deselect()
+
+	double get_selection_weight()
 	{
-		selected = false;
+		return selection_weight;
 	}
-	
-	bool selected;
+
+	void set_selection_weight(const double Weight)
+	{
+		selection_weight = Weight;
+	}
+
+	bool visible_selection;
+	double selection_weight;
 };
 
 } // namespace k3d
 
 #endif // !K3DSDK_SELECTABLE_H
+
+
