@@ -25,10 +25,7 @@
 		\author Tim Shead (tshead@k-3d.com)
 */
 
-#ifndef SDPSL_H
-#include <sdpsl/sdpsl.h>
-#endif // !SDPSL_H
-
+#include "sl.h"
 #include <memory>
 
 // Forward declarations
@@ -38,10 +35,10 @@ namespace k3d
 {
 
 /// Recompiles a shader from source iff it's out-of-date
-bool compile_shader(const boost::filesystem::path& SourceFile, const std::string RenderEngineType, const std::string RenderEngine);
+bool compile_shader(const boost::filesystem::path& SourceFile, const std::string& RenderEngineType, const std::string& RenderEngine);
 
 /// Returns a shader by name (could return NULL)
-std::auto_ptr<sdpsl::shader> lookup_shader(const std::string ShaderName);
+std::auto_ptr<sl::shader> lookup_shader(const std::string& ShaderName);
 
 } // namespace k3d
 
