@@ -1,7 +1,7 @@
 // K-3D
-// Copyright (c) 1995-2004, Timothy M. Shead
+// Copyright (c) 2004-2005, Romain Behar
 //
-// Contact: tshead@k-3d.com
+// Contact: romainbehar@yahoo.com
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public
@@ -22,31 +22,27 @@
 		\author Romain Behar (romainbehar@yahoo.com)
 */
 
-#include <k3dsdk/classes.h>
 #include <k3dsdk/color.h>
 #include <k3dsdk/file_helpers.h>
 #include <k3dsdk/ideletable.h>
 #include <k3dsdk/idocument.h>
 #include <k3dsdk/ifile_format.h>
-#include <k3dsdk/igeometry_read_format.h>
+#include <k3dsdk/idocument_read_format.h>
 #include <k3dsdk/imaterial.h>
-#include <k3dsdk/imaterial_collection.h>
-#include <k3dsdk/iobject.h>
-#include <k3dsdk/iobject_collection.h>
+#include <k3dsdk/inode.h>
+#include <k3dsdk/inode_collection.h>
 #include <k3dsdk/log.h>
 #include <k3dsdk/material.h>
 #include <k3dsdk/mesh.h>
 #include <k3dsdk/module.h>
-#include <k3dsdk/plugins.h>
 #include <k3dsdk/result.h>
-#include <k3dsdk/selection.h>
 #include <k3dsdk/string_modifiers.h>
 #include <k3dsdk/utility.h>
 #include <k3dsdk/vectors.h>
 
 #include "helpers.h"
 
-#include <boost/filesystem/fstream.hpp>
+#include <k3dsdk/fstream.h>
 #include <boost/filesystem/path.hpp>
 
 namespace libk3dgeometry
@@ -57,13 +53,13 @@ namespace libk3dgeometry
 
 class mfx_reader_implementation :
 	public k3d::ifile_format,
-	public k3d::igeometry_read_format,
+	public k3d::idocument_read_format,
 	public k3d::ideletable
 {
 public:
 	unsigned long priority()
 	{
-		return 0;
+		return 128;
 	}
 
 	bool query_can_handle(const boost::filesystem::path& FilePath)
@@ -72,16 +68,6 @@ public:
 		return "mfx" == ext;
 	}
 
-	bool pre_read(k3d::idocument& Document, const boost::filesystem::path& FilePath)
-	{
-		return true;
-	}
-
-	bool read_options(k3d::idocument& Document, const boost::filesystem::path& FilePath)
-	{
-		return true;
-	}
-
 	bool read_file(k3d::idocument& Document, const boost::filesystem::path& FilePath);
 
 	k3d::iplugin_factory& factory()
@@ -91,7 +77,7 @@ public:
 
 	static k3d::iplugin_factory& get_factory()
 	{
-		static k3d::plugin_factory<k3d::application_plugin<mfx_reader_implementation>, k3d::interface_list<k3d::igeometry_read_format> > factory(
+		static k3d::plugin_factory<k3d::application_plugin<mfx_reader_implementation>, k3d::interface_list<k3d::idocument_read_format> > factory(
 			k3d::uuid(0xfa7c6bb0, 0x41ab44b3, 0xae92f484, 0xdb0f1e9b),
 			"MFXReader",
 			"OpenFX Model ( .mfx )",
@@ -143,8 +129,8 @@ private:
 	file_format_t m_CurrentFormat;
 
 	// Mesh variables
-	k3d::iobject* m_current_mesh_object;
-	k3d::iobject* m_current_mesh_instance;
+	k3d::inode* m_current_mesh_object;
+	k3d::inode* m_current_mesh_instance;
 	k3d::mesh* m_current_mesh;
 	k3d::imaterial* default_material;
 
@@ -164,8 +150,10 @@ private:
 
 bool mfx_reader_implementation::read_file(k3d::idocument& Document, const boost::filesystem::path& FilePath)
 {
+	k3d::log() << info << "Reading " << FilePath.native_file_string() << " with " << factory().name() << std::endl;
+
 	// Allocate a buffer and load file
-	boost::filesystem::ifstream file(FilePath, std::ios::in | std::ios::binary);
+	k3d::filesystem::ifstream file(FilePath, std::ios::in | std::ios::binary);
 	return_val_if_fail(file.good(), false);
 	file.seekg(0, std::ios::end);
 	m_BufSize = file.tellg();
@@ -220,7 +208,7 @@ bool mfx_reader_implementation::GetChunkInfo()
 {
 	if(m_ChunkLength)
 		{
-			std::cerr << debug << "Previous chunk not empty, remaining : " << m_ChunkLength << std::endl;
+			k3d::log() << debug << "Previous chunk not empty, remaining : " << m_ChunkLength << std::endl;
 		}
 
 	// Get chunk ID and chunk length
@@ -229,7 +217,7 @@ bool mfx_reader_implementation::GetChunkInfo()
 
 	m_CurrentChunkId = GetChunkId();
 	m_ChunkLength = GetLong();
-	std::cerr << debug << "Chunk ID : " << m_CurrentChunkId << ", length : " << m_ChunkLength << std::endl;
+	k3d::log() << debug << "Chunk ID : " << m_CurrentChunkId << ", length : " << m_ChunkLength << std::endl;
 
 	return true;
 }
@@ -245,7 +233,7 @@ bool mfx_reader_implementation::GetSubchunkInfo()
 
 	m_CurrentSubchunkId = GetChunkId();
 	m_SubchunkLength = static_cast<unsigned long>(GetShort());
-	std::cerr << debug << "Sub-chunk " << m_CurrentSubchunkId << ", length = " << m_SubchunkLength << std::endl;
+	k3d::log() << debug << "Sub-chunk " << m_CurrentSubchunkId << ", length = " << m_SubchunkLength << std::endl;
 
 	return true;
 }
@@ -304,7 +292,7 @@ bool mfx_reader_implementation::load_model(k3d::idocument& Document)
 				read_skeleton(2);
 			else
 				{
-					std::cerr << debug << "Skipping .mfx chunk " << m_CurrentChunkId << std::endl;
+					k3d::log() << debug << "Skipping .mfx chunk " << m_CurrentChunkId << std::endl;
 					SkipChunk();
 				}
 
@@ -346,7 +334,7 @@ unsigned short mfx_reader_implementation::GetShort()
 
 unsigned long mfx_reader_implementation::GetLong()
 {
-	// TODO: implement for big endian
+	/** \todo implement for big endian */
 	assert_warning(k3d::little_endian());
 
 	if(m_Ptr+4 <= m_Buf+m_BufSize)
@@ -364,7 +352,7 @@ unsigned long mfx_reader_implementation::GetLong()
 		}
 	else
 		{
-			std::cerr << k3d::error << "Unexpected end of file." << std::endl;
+			k3d::log() << k3d::error << "Unexpected end of file." << std::endl;
 			return 0;
 		}
 }
@@ -520,7 +508,6 @@ bool mfx_reader_implementation::read_faces(bool short_form)
 
 	k3d::polyhedron* const polyhedron = new k3d::polyhedron();
 	return_val_if_fail(polyhedron, false);
-	polyhedron->material = default_material;
 	m_current_mesh->polyhedra.push_back(polyhedron);
 
 	for(unsigned long i = 0; i < new_faces; i++)
@@ -553,7 +540,7 @@ bool mfx_reader_implementation::read_faces(bool short_form)
 //std::cout << " face params : " << attrib << " " << param << " " << map << " " << shader << " " << axis << pad << " | " << color << " , " << matcolor << std::endl;
 
 			// Create a triangle ...
-			k3d::polyhedron::edges_t edges;
+			std::vector<k3d::split_edge*> edges;
 			assert_warning(f1 < m_CurrentPointList.size());
 			assert_warning(f2 < m_CurrentPointList.size());
 			assert_warning(f3 < m_CurrentPointList.size());
@@ -562,9 +549,8 @@ bool mfx_reader_implementation::read_faces(bool short_form)
 			edges.push_back(new k3d::split_edge(m_CurrentPointList[f3]));
 
 			k3d::loop_edges(edges.begin(), edges.end());
-			polyhedron->edges.insert(polyhedron->edges.end(), edges.begin(), edges.end());
 
-			k3d::face* const face = new k3d::face(*edges.begin());
+			k3d::face* const face = new k3d::face(*edges.begin(), default_material);
 			return_val_if_fail(face, false);
 
 			polyhedron->faces.push_back(face);
@@ -594,7 +580,7 @@ bool mfx_reader_implementation::read_skeleton(unsigned int version)
 		{
 			// Load name
 			std::string name = GetString(16);
-			std::cerr << debug << "Skeleton root : " << name << std::endl;
+			k3d::log() << debug << "Skeleton root : " << name << std::endl;
 		}
 
 	std::vector<double> u;
@@ -639,7 +625,7 @@ bool mfx_reader_implementation::read_skeleton(unsigned int version)
 				{
 					// Load name
 					std::string name = GetString(16);
-					std::cerr << debug << "Skeleton part : " << name << std::endl;
+					k3d::log() << debug << "Skeleton part : " << name << std::endl;
 				}
 
 			std::vector<double> u;
