@@ -1,7 +1,7 @@
 // K-3D
-// Copyright (c) 1995-2004, Timothy M. Shead
+// Copyright (c) 2004-2005, Timothy M. Shead
 //
-// Contact: tshead@k-3d.com
+// Contact: romainbehar@yahoo.com
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public
@@ -22,30 +22,30 @@
 		\author Romain Behar (romainbehar@yahoo.com)
 */
 
+#include <k3dsdk/i18n.h>
 #include <k3dsdk/algebra.h>
 #include <k3dsdk/classes.h>
 #include <k3dsdk/color.h>
+#include <k3dsdk/create_plugins.h>
 #include <k3dsdk/file_helpers.h>
 #include <k3dsdk/idag.h>
 #include <k3dsdk/ideletable.h>
 #include <k3dsdk/idocument.h>
 #include <k3dsdk/idocument_plugin_factory.h>
 #include <k3dsdk/ifile_format.h>
-#include <k3dsdk/igeometry_read_format.h>
+#include <k3dsdk/idocument_read_format.h>
 #include <k3dsdk/imaterial.h>
-#include <k3dsdk/imaterial_collection.h>
-#include <k3dsdk/iobject.h>
-#include <k3dsdk/iobject_collection.h>
+#include <k3dsdk/inode.h>
+#include <k3dsdk/inode_collection.h>
 #include <k3dsdk/irenderman.h>
 #include <k3dsdk/itransform_sink.h>
 #include <k3dsdk/itransform_source.h>
 #include <k3dsdk/material.h>
 #include <k3dsdk/mesh.h>
 #include <k3dsdk/module.h>
-#include <k3dsdk/plugins.h>
 #include <k3dsdk/property.h>
 #include <k3dsdk/result.h>
-#include <k3dsdk/selection.h>
+#include <k3dsdk/string_cast.h>
 #include <k3dsdk/string_modifiers.h>
 #include <k3dsdk/utility.h>
 #include <k3dsdk/vectors.h>
@@ -55,7 +55,7 @@
 #include <Hapy/Parser.h>
 #include <Hapy/Rules.h>
 
-#include <boost/filesystem/fstream.hpp>
+#include <k3dsdk/fstream.h>
 #include <boost/filesystem/path.hpp>
 
 #include <iostream>
@@ -249,7 +249,7 @@ void create_parser()
 		| "Points" >> rtparameters
 		| "PointsPolygons" >> rtint_array >> rtint_array >> rtparameters
 		| "PointsGeneralPolygons" >> rtint_array >> rtint_array >> rtint_array >> rtparameters
-		// TODO RiBasis
+		/** \todo RiBasis */
 		| "Patch" >> rtstring >> rtparameters
 		| "PatchMesh" >> rtstring >> rtint >> rtstring >> rtint >> rtstring >> rtparameters
 		| "NuPatch" >> rtint >> rtint >> rtfloat_array >> rtfloat >> rtfloat >> rtint >> rtint >> rtfloat_array >> rtfloat >> rtfloat >> rtparameters
@@ -320,7 +320,7 @@ typedef std::vector<k3d::vector3> coordinates_t;
 
 class rib_reader_implementation :
 	public k3d::ifile_format,
-	public k3d::igeometry_read_format,
+	public k3d::idocument_read_format,
 	public k3d::ideletable
 {
 public:
@@ -344,7 +344,7 @@ public:
 
 	unsigned long priority()
 	{
-		return 0;
+		return 128;
 	}
 
 	bool query_can_handle(const boost::filesystem::path& FilePath)
@@ -352,16 +352,6 @@ public:
 		return "rib" == k3d::file_extension(FilePath);
 	}
 
-	bool pre_read(k3d::idocument& Document, const boost::filesystem::path& FilePath)
-	{
-		return true;
-	}
-
-	bool read_options(k3d::idocument& Document, const boost::filesystem::path& FilePath)
-	{
-		return true;
-	}
-
 	bool read_file(k3d::idocument& Document, const boost::filesystem::path& FilePath);
 
 	k3d::iplugin_factory& factory()
@@ -371,11 +361,11 @@ public:
 
 	static k3d::iplugin_factory& get_factory()
 	{
-		static k3d::plugin_factory<k3d::application_plugin<rib_reader_implementation>, k3d::interface_list<k3d::igeometry_read_format> > factory(
+		static k3d::plugin_factory<k3d::application_plugin<rib_reader_implementation>, k3d::interface_list<k3d::idocument_read_format> > factory(
 			k3d::uuid(0x9a392c01, 0x50234b23, 0xa61245ff, 0x9345ce15),
 			"RIBReader",
-			"RenderMan ( .rib )",
-			"");
+			_("RenderMan ( .rib )"),
+			"GeometryReader");
 
 		return factory;
 	}
@@ -411,17 +401,17 @@ private:
 	std::string m_solid_type;
 
 	// Mesh variables and functions
-	std::stack<k3d::iobject*> m_transform_objects;
+	std::stack<k3d::inode*> m_transform_objects;
 	k3d::mesh* m_current_mesh;
 	std::string m_current_name;
-	k3d::iobject* m_current_mesh_object;
-	k3d::iobject* m_current_mesh_instance;
+	k3d::inode* m_current_mesh_object;
+	k3d::inode* m_current_mesh_instance;
 	k3d::nupatch* m_current_nupatch;
 	k3d::bilinear_patch* m_current_bilinear_patch;
 	k3d::polyhedron* m_current_polyhedron;
 	k3d::linear_curve_group* m_current_linear_curve_group;
-	k3d::iobject* m_current_material;
-	k3d::iobject* m_current_shader;
+	k3d::inode* m_current_material;
+	k3d::inode* m_current_shader;
 
 	// Save new polyhedra to set companions
 	std::vector<k3d::polyhedron*> m_polyhedra;
@@ -493,11 +483,11 @@ private:
 		return true;
 	}
 
-	bool create_polygon(coordinates_t& points, k3d::mesh* mesh)
+	bool create_polygon(coordinates_t& points, k3d::mesh* mesh, k3d::idocument& Document)
 	{
 		return_val_if_fail(mesh, false);
 
-		k3d::polyhedron::edges_t edges;
+		std::vector<k3d::split_edge*> edges;
 		coordinates_t::const_iterator point = points.begin();
 		for(; point != points.end(); point++)
 			{
@@ -514,9 +504,7 @@ private:
 		return_val_if_fail(polyhedron, false);
 		m_polyhedra.push_back(polyhedron);
 
-		polyhedron->edges.insert(polyhedron->edges.end(), edges.begin(), edges.end());
-
-		k3d::face* const face = new k3d::face(*edges.begin());
+		k3d::face* const face = new k3d::face(*edges.begin(), assign_current_material(Document));
 		return_val_if_fail(face, false);
 
 		polyhedron->faces.push_back(face);
@@ -527,22 +515,22 @@ private:
 
 	bool create_transform(k3d::idocument& Document)
 	{
-		k3d::iobject* const transform = k3d::create_document_plugin(k3d::classes::Transform(), Document);
+		k3d::inode* const transform = k3d::create_plugin<k3d::inode>(k3d::classes::Transform(), Document);
 		return_val_if_fail(transform, false);
 
-		transform->set_name("Transform " + k3d::to_string(++m_transform_number));
+		transform->set_name("Transform " + k3d::string_cast(++m_transform_number));
 		connect_transforms(transform, Document);
 		m_transform_objects.push(transform);
 
 		return true;
 	}
 
-	bool connect_transforms(k3d::iobject* Object, k3d::idocument& Document)
+	bool connect_transforms(k3d::inode* Object, k3d::idocument& Document)
 	{
 		if(m_transform_objects.empty())
 			return false;
 
-		k3d::iobject* const last_transform = m_transform_objects.top();
+		k3d::inode* const last_transform = m_transform_objects.top();
 		k3d::itransform_source* const transform_source = dynamic_cast<k3d::itransform_source*>(last_transform);
 		return_val_if_fail(transform_source, false);
 
@@ -639,14 +627,14 @@ private:
 			k3d::vector4(v[8], v[9], v[10], v[11]),
 			k3d::vector4(v[12], v[13], v[14], v[15]));
 
-		Matrix = Matrix.Transpose();
+		Matrix = k3d::transpose(Matrix);
 	}
 
-	void set_current_transformations(k3d::iobject* Object)
+	void set_current_transformations(k3d::inode* Object)
 	{
-		assert_warning(k3d::set_property_value(*Object, "position", k3d::vector3(*m_translations.rbegin() + extractTranslation(*m_transformations.rbegin()))));
-		assert_warning(k3d::set_property_value(*Object, "orientation", k3d::angle_axis(k3d::rotation3D(*m_rotations.rbegin() * *m_transformations.rbegin()))));
-		assert_warning(k3d::set_property_value(*Object, "scale", k3d::vector3(mult_scales(*m_scalings.rbegin(), k3d::extractScaling(*m_transformations.rbegin())))));
+		assert_warning(k3d::set_value(*Object, "position", k3d::vector3(*m_translations.rbegin() + extract_translation(*m_transformations.rbegin()))));
+		assert_warning(k3d::set_value(*Object, "orientation", k3d::angle_axis(k3d::rotation3D(*m_rotations.rbegin() * *m_transformations.rbegin()))));
+		assert_warning(k3d::set_value(*Object, "scale", k3d::vector3(mult_scales(*m_scalings.rbegin(), k3d::deprecated_extract_scaling(*m_transformations.rbegin())))));
 	}
 
 	void rib_stack_push(const std::string& name, k3d::idocument& Document)
@@ -706,7 +694,7 @@ private:
 			m_inside_world = false;
 	}
 
-	void set_object_properties(k3d::iobject* Object, const std::string& Name, k3d::idocument& Document)
+	void set_object_properties(k3d::inode* Object, const std::string& Name, k3d::idocument& Document)
 	{
 		assert_warning(Object);
 
@@ -728,14 +716,14 @@ private:
 		First += 4;
 		m[3] = k3d::vector4(First[0], First[1], First[2], First[3]);
 
-		return m.Transpose();
+		return k3d::transpose(m);
 	}
 
 	// Debug
 	void Childs(const Hapy::Pree& Node)
 	{
 		for(Hapy::Pree::const_iterator node = Node.begin(); node != Node.end(); node++)
-			std::cerr << debug << "Node child : " << node->rid() << " -> " << node->image() << std::endl;
+			k3d::log() << debug << "Node child : " << node->rid() << " -> " << node->image() << std::endl;
 	}
 };
 
@@ -1046,13 +1034,13 @@ void rib_reader_implementation::parse_subpree(const Hapy::Pree& Node, k3d::idocu
 						}
 					}
 					else
-						std::cerr << debug << "RIB reader : unsupported point type for bilinear patch" << std::endl;
+						k3d::log() << debug << "RIB reader : unsupported point type for bilinear patch" << std::endl;
 				}
 				else if("bicubic" == type)
 				{
 				}
 				else
-					std::cerr << debug << "Invalid RenderMan Patch type " << type << std::endl;
+					k3d::log() << debug << "Invalid RenderMan Patch type " << type << std::endl;
 
 				// Set material and transform properties ...
 				m_current_bilinear_patch->material = assign_current_material(Document);
@@ -1074,7 +1062,7 @@ void rib_reader_implementation::parse_subpree(const Hapy::Pree& Node, k3d::idocu
 						coordinates_t points;
 						get_rtfloat_array(float_array, points);
 
-						create_polygon(points, m_current_mesh);
+						create_polygon(points, m_current_mesh, Document);
 						set_object_properties(m_current_mesh_instance, "Polygon", Document);
 					}
 				//else
@@ -1150,7 +1138,7 @@ void rib_reader_implementation::parse_subpree(const Hapy::Pree& Node, k3d::idocu
 
 								if(!face)
 									{
-										face = new k3d::face(edge);
+										face = new k3d::face(edge, assign_current_material(Document));
 										return_if_fail(face);
 										m_current_polyhedron->faces.push_back(face);
 									}
@@ -1159,7 +1147,6 @@ void rib_reader_implementation::parse_subpree(const Hapy::Pree& Node, k3d::idocu
 										previous_edge->face_clockwise = edge;
 									}
 
-								m_current_polyhedron->edges.push_back(edge);
 								previous_edge = edge;
 							}
 
@@ -1171,101 +1158,100 @@ void rib_reader_implementation::parse_subpree(const Hapy::Pree& Node, k3d::idocu
 					}
 				}
 
-				// Set material and transform properties ...
-				m_current_polyhedron->material = assign_current_material(Document);
+				// Set transform properties ...
 				set_object_properties(m_current_mesh_instance, "Polygons", Document);
 			}
 			else if("Cone" == (*pir).image())
 			{
-				k3d::iobject* const cone = k3d::create_document_plugin(k3d::classes::Cone(), Document);
+				k3d::inode* const cone = k3d::create_plugin<k3d::inode>(k3d::classes::Cone(), Document);
 				assert_warning(cone);
 
-				k3d::set_property_value(*cone, "height", k3d::from_string<double>((++pir)->image(), 0));
-				k3d::set_property_value(*cone, "radius", k3d::from_string<double>((++pir)->image(), 0));
-				k3d::set_property_value(*cone, "thetamax", k3d::radians(k3d::from_string<double>((++pir)->image(), 0)));
+				k3d::set_value(*cone, "height", k3d::from_string<double>((++pir)->image(), 0));
+				k3d::set_value(*cone, "radius", k3d::from_string<double>((++pir)->image(), 0));
+				k3d::set_value(*cone, "thetamax", k3d::radians(k3d::from_string<double>((++pir)->image(), 0)));
 
-				k3d::set_property_value(*cone, "material", assign_current_material(Document));
+				k3d::set_value(*cone, "material", assign_current_material(Document));
 				set_object_properties(cone, "Cone", Document);
 			}
 			else if("Cylinder" == (*pir).image())
 			{
-				k3d::iobject* const cylinder = k3d::create_document_plugin(k3d::classes::Cylinder(), Document);
+				k3d::inode* const cylinder = k3d::create_plugin<k3d::inode>(k3d::classes::Cylinder(), Document);
 				assert_warning(cylinder);
 
-				k3d::set_property_value(*cylinder, "radius", k3d::from_string<double>((++pir)->image(), 0));
-				k3d::set_property_value(*cylinder, "zmin", k3d::from_string<double>((++pir)->image(), 0));
-				k3d::set_property_value(*cylinder, "zmax", k3d::from_string<double>((++pir)->image(), 0));
-				k3d::set_property_value(*cylinder, "thetamax", k3d::radians(k3d::from_string<double>((++pir)->image(), 0)));
+				k3d::set_value(*cylinder, "radius", k3d::from_string<double>((++pir)->image(), 0));
+				k3d::set_value(*cylinder, "zmin", k3d::from_string<double>((++pir)->image(), 0));
+				k3d::set_value(*cylinder, "zmax", k3d::from_string<double>((++pir)->image(), 0));
+				k3d::set_value(*cylinder, "thetamax", k3d::radians(k3d::from_string<double>((++pir)->image(), 0)));
 
-				k3d::set_property_value(*cylinder, "material", assign_current_material(Document));
+				k3d::set_value(*cylinder, "material", assign_current_material(Document));
 				set_object_properties(cylinder, "Cylinder", Document);
 			}
 			else if("Disk" == (*pir).image())
 			{
-				k3d::iobject* const disk = k3d::create_document_plugin(k3d::classes::Disk(), Document);
+				k3d::inode* const disk = k3d::create_plugin<k3d::inode>(k3d::classes::Disk(), Document);
 				assert_warning(disk);
 
-				k3d::set_property_value(*disk, "height", k3d::from_string<double>((++pir)->image(), 0));
-				k3d::set_property_value(*disk, "radius", k3d::from_string<double>((++pir)->image(), 0));
-				k3d::set_property_value(*disk, "thetamax", k3d::radians(k3d::from_string<double>((++pir)->image(), 0)));
+				k3d::set_value(*disk, "height", k3d::from_string<double>((++pir)->image(), 0));
+				k3d::set_value(*disk, "radius", k3d::from_string<double>((++pir)->image(), 0));
+				k3d::set_value(*disk, "thetamax", k3d::radians(k3d::from_string<double>((++pir)->image(), 0)));
 
-				k3d::set_property_value(*disk, "material", assign_current_material(Document));
+				k3d::set_value(*disk, "material", assign_current_material(Document));
 				set_object_properties(disk, "Disk", Document);
 			}
 			else if("Hyperboloid" == (*pir).image())
 			{
-				k3d::iobject* const hyperboloid = k3d::create_document_plugin(k3d::classes::Hyperboloid(), Document);
+				k3d::inode* const hyperboloid = k3d::create_plugin<k3d::inode>(k3d::classes::Hyperboloid(), Document);
 				assert_warning(hyperboloid);
 
-				k3d::set_property_value(*hyperboloid, "p1x", k3d::from_string<double>((++pir)->image(), 0));
-				k3d::set_property_value(*hyperboloid, "p1y", k3d::from_string<double>((++pir)->image(), 0));
-				k3d::set_property_value(*hyperboloid, "p1z", k3d::from_string<double>((++pir)->image(), 0));
-				k3d::set_property_value(*hyperboloid, "p2x", k3d::from_string<double>((++pir)->image(), 0));
-				k3d::set_property_value(*hyperboloid, "p2y", k3d::from_string<double>((++pir)->image(), 0));
-				k3d::set_property_value(*hyperboloid, "p2z", k3d::from_string<double>((++pir)->image(), 0));
-				k3d::set_property_value(*hyperboloid, "thetamax", k3d::radians(k3d::from_string<double>((++pir)->image(), 0)));
+				k3d::set_value(*hyperboloid, "p1x", k3d::from_string<double>((++pir)->image(), 0));
+				k3d::set_value(*hyperboloid, "p1y", k3d::from_string<double>((++pir)->image(), 0));
+				k3d::set_value(*hyperboloid, "p1z", k3d::from_string<double>((++pir)->image(), 0));
+				k3d::set_value(*hyperboloid, "p2x", k3d::from_string<double>((++pir)->image(), 0));
+				k3d::set_value(*hyperboloid, "p2y", k3d::from_string<double>((++pir)->image(), 0));
+				k3d::set_value(*hyperboloid, "p2z", k3d::from_string<double>((++pir)->image(), 0));
+				k3d::set_value(*hyperboloid, "thetamax", k3d::radians(k3d::from_string<double>((++pir)->image(), 0)));
 
-				k3d::set_property_value(*hyperboloid, "material", assign_current_material(Document));
+				k3d::set_value(*hyperboloid, "material", assign_current_material(Document));
 				set_object_properties(hyperboloid, "Hyperboloid", Document);
 			}
 			else if("Paraboloid" == (*pir).image())
 			{
-				k3d::iobject* const paraboloid = k3d::create_document_plugin(k3d::classes::Paraboloid(), Document);
+				k3d::inode* const paraboloid = k3d::create_plugin<k3d::inode>(k3d::classes::Paraboloid(), Document);
 				assert_warning(paraboloid);
 
-				k3d::set_property_value(*paraboloid, "radius", k3d::from_string<double>((++pir)->image(), 0));
-				k3d::set_property_value(*paraboloid, "zmin", k3d::from_string<double>((++pir)->image(), 0));
-				k3d::set_property_value(*paraboloid, "zmax", k3d::from_string<double>((++pir)->image(), 0));
-				k3d::set_property_value(*paraboloid, "thetamax", k3d::radians(k3d::from_string<double>((++pir)->image(), 0)));
+				k3d::set_value(*paraboloid, "radius", k3d::from_string<double>((++pir)->image(), 0));
+				k3d::set_value(*paraboloid, "zmin", k3d::from_string<double>((++pir)->image(), 0));
+				k3d::set_value(*paraboloid, "zmax", k3d::from_string<double>((++pir)->image(), 0));
+				k3d::set_value(*paraboloid, "thetamax", k3d::radians(k3d::from_string<double>((++pir)->image(), 0)));
 
-				k3d::set_property_value(*paraboloid, "material", assign_current_material(Document));
+				k3d::set_value(*paraboloid, "material", assign_current_material(Document));
 				set_object_properties(paraboloid, "Paraboloid", Document);
 			}
 			else if("Sphere" == (*pir).image())
 			{
-				k3d::iobject* const sphere = k3d::create_document_plugin(k3d::classes::Sphere(), Document);
+				k3d::inode* const sphere = k3d::create_plugin<k3d::inode>(k3d::classes::Sphere(), Document);
 				assert_warning(sphere);
 
-				k3d::set_property_value(*sphere, "radius", k3d::from_string<double>((++pir)->image(), 0));
-				k3d::set_property_value(*sphere, "zmin", k3d::from_string<double>((++pir)->image(), 0));
-				k3d::set_property_value(*sphere, "zmax", k3d::from_string<double>((++pir)->image(), 0));
-				k3d::set_property_value(*sphere, "thetamax", k3d::radians(k3d::from_string<double>((++pir)->image(), 0)));
+				k3d::set_value(*sphere, "radius", k3d::from_string<double>((++pir)->image(), 0));
+				k3d::set_value(*sphere, "zmin", k3d::from_string<double>((++pir)->image(), 0));
+				k3d::set_value(*sphere, "zmax", k3d::from_string<double>((++pir)->image(), 0));
+				k3d::set_value(*sphere, "thetamax", k3d::radians(k3d::from_string<double>((++pir)->image(), 0)));
 
-				k3d::set_property_value(*sphere, "material", assign_current_material(Document));
+				k3d::set_value(*sphere, "material", assign_current_material(Document));
 				set_object_properties(sphere, "Sphere", Document);
 			}
 			else if("Torus" == (*pir).image())
 			{
-				k3d::iobject* const torus = k3d::create_document_plugin(k3d::classes::Torus(), Document);
+				k3d::inode* const torus = k3d::create_plugin<k3d::inode>(k3d::classes::Torus(), Document);
 				assert_warning(torus);
 
-				k3d::set_property_value(*torus, "majorradius", k3d::from_string<double>((++pir)->image(), 0));
-				k3d::set_property_value(*torus, "minorradius", k3d::from_string<double>((++pir)->image(), 0));
-				k3d::set_property_value(*torus, "phimin", k3d::radians(k3d::from_string<double>((++pir)->image(), 0)));
-				k3d::set_property_value(*torus, "phimax", k3d::radians(k3d::from_string<double>((++pir)->image(), 0)));
-				k3d::set_property_value(*torus, "thetamax", k3d::radians(k3d::from_string<double>((++pir)->image(), 0)));
+				k3d::set_value(*torus, "majorradius", k3d::from_string<double>((++pir)->image(), 0));
+				k3d::set_value(*torus, "minorradius", k3d::from_string<double>((++pir)->image(), 0));
+				k3d::set_value(*torus, "phimin", k3d::radians(k3d::from_string<double>((++pir)->image(), 0)));
+				k3d::set_value(*torus, "phimax", k3d::radians(k3d::from_string<double>((++pir)->image(), 0)));
+				k3d::set_value(*torus, "thetamax", k3d::radians(k3d::from_string<double>((++pir)->image(), 0)));
 
-				k3d::set_property_value(*torus, "material", assign_current_material(Document));
+				k3d::set_value(*torus, "material", assign_current_material(Document));
 				set_object_properties(torus, "Torus", Document);
 			}
 			else if("Blobby" == (*pir).image())
@@ -1520,11 +1506,11 @@ void rib_reader_implementation::parse_subpree(const Hapy::Pree& Node, k3d::idocu
 				std::string name = no_quotes((++shader_node)->image());
 				/*int val = */k3d::from_string<int>((++shader_node)->image(), 0);
 
-				m_current_shader = create_document_plugin(k3d::classes::RenderManLightShader(), Document, "Light shader " + name);
-				m_current_material = create_document_plugin(k3d::classes::RenderManLight(), Document, "Light " + name);
-				assert_warning(k3d::set_property_value(*m_current_material, "shader", m_current_shader));
+				m_current_shader = k3d::create_plugin<k3d::inode>(k3d::classes::RenderManLightShader(), Document, "Light shader " + name);
+				m_current_material = k3d::create_plugin<k3d::inode>(k3d::classes::RenderManLight(), Document, "Light " + name);
+				assert_warning(k3d::set_value(*m_current_material, "shader", m_current_shader));
 
-				k3d::set_property_value(*m_current_shader, "shader_name", name);
+				k3d::set_value(*m_current_shader, "shader_name", name);
 
 				// rtparameters
 				const Hapy::Pree& parameters = *++shader_node;
@@ -1539,22 +1525,22 @@ void rib_reader_implementation::parse_subpree(const Hapy::Pree& Node, k3d::idocu
 					if(m_declarations[variable] == "float")
 					{
 						const Hapy::Pree& float_val = *(value.begin() + 1);
-						k3d::set_property_value(*m_current_shader, variable, k3d::from_string<double>(float_val.image(), 0));
+						k3d::set_value(*m_current_shader, variable, k3d::from_string<double>(float_val.image(), 0));
 					}
 					else
 					if(m_declarations[variable] == "color")
 					{
 						const Hapy::Pree& float_vals = *(value.begin() + 1);
-						k3d::set_property_value(*m_current_shader, variable, k3d::from_string<k3d::color>(float_vals.image(), k3d::color(1.0, 1.0, 1.0)));
+						k3d::set_value(*m_current_shader, variable, k3d::from_string<k3d::color>(float_vals.image(), k3d::color(1.0, 1.0, 1.0)));
 					}
 					else
 					if(m_declarations[variable] == "point")
 					{
 						const Hapy::Pree& float_vals = *(value.begin() + 1);
-						k3d::set_property_value(*m_current_shader, variable, k3d::from_string<k3d::vector3>(float_vals.image(), k3d::vector3(0.0, 0.0, 0.0)));
+						k3d::set_value(*m_current_shader, variable, k3d::from_string<k3d::vector3>(float_vals.image(), k3d::vector3(0.0, 0.0, 0.0)));
 					}
 					else
-						std::cerr << debug << "Variable type not declared for " << variable << " : " << m_declarations[variable] << std::endl;
+						k3d::log() << debug << "Variable type not declared for " << variable << " : " << m_declarations[variable] << std::endl;
 				}
 			}
 			else
@@ -1563,14 +1549,14 @@ void rib_reader_implementation::parse_subpree(const Hapy::Pree& Node, k3d::idocu
 				std::string name = no_quotes((++shader_node)->image());
 				if(name != "null")
 				{
-					m_current_shader = create_document_plugin(k3d::classes::RenderManSurfaceShader(), Document, "Surface shader " + name);
-					m_current_material = create_document_plugin(k3d::classes::RenderManMaterial(), Document, "Surface material " + name);
-					assert_warning(k3d::set_property_value(*m_current_material, "surface_shader", m_current_shader));
+					m_current_shader = k3d::create_plugin<k3d::inode>(k3d::classes::RenderManSurfaceShader(), Document, "Surface shader " + name);
+					m_current_material = k3d::create_plugin<k3d::inode>(k3d::classes::RenderManMaterial(), Document, "Surface material " + name);
+					assert_warning(k3d::set_value(*m_current_material, "surface_shader", m_current_shader));
 
-					k3d::set_property_value(*m_current_shader, "shader_name", name);
-					k3d::set_property_value(*m_current_material, "color", m_current_color);
-					k3d::set_property_value(*m_current_material, "opacity", m_current_opacity);
-					k3d::set_property_value(*m_current_material, "matte", m_current_matte ? true : false);
+					k3d::set_value(*m_current_shader, "shader_name", name);
+					k3d::set_value(*m_current_material, "color", m_current_color);
+					k3d::set_value(*m_current_material, "opacity", m_current_opacity);
+					k3d::set_value(*m_current_material, "matte", m_current_matte ? true : false);
 
 					// rtparameters
 					const Hapy::Pree& parameters = *++shader_node;
@@ -1585,16 +1571,16 @@ void rib_reader_implementation::parse_subpree(const Hapy::Pree& Node, k3d::idocu
 						if(m_declarations[variable] == "float")
 						{
 							const Hapy::Pree& float_val = *(value.begin() + 1);
-							k3d::set_property_value(*m_current_shader, variable, k3d::from_string<double>(float_val.image(), 0));
+							k3d::set_value(*m_current_shader, variable, k3d::from_string<double>(float_val.image(), 0));
 						}
 						else
 						if(m_declarations[variable] == "color")
 						{
 							const Hapy::Pree& float_vals = *(value.begin() + 1);
-							k3d::set_property_value(*m_current_shader, variable, k3d::from_string<k3d::color>(float_vals.image(), k3d::color(1.0, 1.0, 1.0)));
+							k3d::set_value(*m_current_shader, variable, k3d::from_string<k3d::color>(float_vals.image(), k3d::color(1.0, 1.0, 1.0)));
 						}
 						else
-							std::cerr << debug << "Variable type not declared for " << variable << " : " << m_declarations[variable] << std::endl;
+							k3d::log() << debug << "Variable type not declared for " << variable << " : " << m_declarations[variable] << std::endl;
 					}
 				}
 			}
@@ -1612,12 +1598,14 @@ void rib_reader_implementation::parse_subpree(const Hapy::Pree& Node, k3d::idocu
 
 bool rib_reader_implementation::read_file(k3d::idocument& Document, const boost::filesystem::path& FilePath)
 {
+	k3d::log() << info << "Reading " << FilePath.native_file_string() << " with " << factory().name() << std::endl;
+
 	// Try to open the input file ...
-	boost::filesystem::ifstream rib_file(FilePath);
+	k3d::filesystem::ifstream rib_file(FilePath);
 	if(!rib_file.good())
 		{
-			std::cerr << warning << __PRETTY_FUNCTION__ << ": error opening [" << FilePath.native_file_string() << "]" << std::endl;
-			return_val_if_fail(0, false);
+			k3d::log() << error << k3d_file_reference << ": error opening [" << FilePath.native_file_string() << "]" << std::endl;
+			return false;
 		}
 
 	std::string buffer("");
@@ -1630,7 +1618,7 @@ bool rib_reader_implementation::read_file(k3d::idocument& Document, const boost:
 
 	if(!parser.parse(buffer))
 	{
-		std::cerr << debug << parser.result().location() << " -> parsing failed" << std::endl;
+		k3d::log() << debug << parser.result().location() << " -> parsing failed" << std::endl;
 		return_val_if_fail(0, false);
 	}
 
