@@ -1,7 +1,7 @@
 // K-3D
-// Copyright (c) 1995-2004, Timothy M. Shead
+// Copyright (c) 2004-2005, Romain Behar
 //
-// Contact: tshead@k-3d.com
+// Contact: romainbehar@yahoo.com
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public
@@ -134,30 +134,30 @@ bool copy_surface(GtsSurface* Surface, k3d::mesh& Mesh)
 		// Determine order of vertices
 		unsigned long p[3];
 		if(edge1_first == edge2_first)
-			{
-				p[0] = edge1_second;
-				p[1] = edge1_first;
-				p[2] = edge2_second;
-			}
+		{
+			p[0] = edge1_second;
+			p[1] = edge1_first;
+			p[2] = edge2_second;
+		}
 		else if(edge1_first == edge2_second)
-			{
-				p[0] = edge1_second;
-				p[1] = edge1_first;
-				p[2] = edge2_first;
-			}
+		{
+			p[0] = edge1_second;
+			p[1] = edge1_first;
+			p[2] = edge2_first;
+		}
 		else if(edge1_second == edge2_first)
-			{
-				p[0] = edge1_first;
-				p[1] = edge1_second;
-				p[2] = edge2_second;
-			}
+		{
+			p[0] = edge1_first;
+			p[1] = edge1_second;
+			p[2] = edge2_second;
+		}
 		else
-			// edge1_second == edge2_second
-			{
-				p[0] = edge1_first;
-				p[1] = edge1_second;
-				p[2] = edge2_first;
-			}
+		// edge1_second == edge2_second
+		{
+			p[0] = edge1_first;
+			p[1] = edge1_second;
+			p[2] = edge2_first;
+		}
 
 		// Create a triangle ...
 		k3d::split_edge* edge1 = new k3d::split_edge(detail::points[p[0]]);
@@ -169,11 +169,7 @@ bool copy_surface(GtsSurface* Surface, k3d::mesh& Mesh)
 		edge3->face_clockwise = edge2;
 		edge2->face_clockwise = edge1;
 
-		polyhedron.edges.push_back(edge1);
-		polyhedron.edges.push_back(edge2);
-		polyhedron.edges.push_back(edge3);
-
-		k3d::face* const face = new k3d::face(edge1);
+		k3d::face* const face = new k3d::face(edge1, 0);
 		return_val_if_fail(face, false);
 
 		polyhedron.faces.push_back(face);
@@ -182,6 +178,8 @@ bool copy_surface(GtsSurface* Surface, k3d::mesh& Mesh)
 	return_val_if_fail(k3d::is_valid(polyhedron), false);
 	//assert_warning(k3d::is_solid(polyhedron));
 
+	k3d::set_companions(polyhedron);
+
 	return true;
 }
 
@@ -239,32 +237,32 @@ bool triangle_test(const k3d::mesh& Mesh)
 {
 	k3d::mesh::polyhedra_t::const_iterator polyhedron;
 	for(polyhedron = Mesh.polyhedra.begin(); polyhedron != Mesh.polyhedra.end(); polyhedron++)
+	{
+		k3d::polyhedron::faces_t::iterator face;
+		for(face = (*polyhedron)->faces.begin(); face != (*polyhedron)->faces.end(); face++)
 		{
-			k3d::polyhedron::faces_t::iterator face;
-			for(face = (*polyhedron)->faces.begin(); face != (*polyhedron)->faces.end(); face++)
-				{
-					k3d::split_edge* first = (*face)->first_edge;
-					// Skip empty faces
-					if(!first)
-						continue;
+			k3d::split_edge* first = (*face)->first_edge;
+			// Skip empty faces
+			if(!first)
+				continue;
 
-					k3d::split_edge* current_edge = first->face_clockwise;
-					// Skip one-edged faces
-					if(!current_edge)
-						continue;
+			k3d::split_edge* current_edge = first->face_clockwise;
+			// Skip one-edged faces
+			if(!current_edge)
+				continue;
 
-					unsigned long edge_number = 1;
+			unsigned long edge_number = 1;
 
-					while(current_edge != first)
-						{
-							edge_number++;
-							current_edge = current_edge->face_clockwise;
-						}
+			while(current_edge != first)
+			{
+				edge_number++;
+				current_edge = current_edge->face_clockwise;
+			}
 
-					if(edge_number != 3)
-						return false;
-				}
+			if(edge_number != 3)
+				return false;
 		}
+	}
 
 	return true;
 }
@@ -291,8 +289,18 @@ GtsSurface* gts_surface(const k3d::mesh& Mesh)
 	// Build the collection of edges ...
 	detail::ordered_edges_t edges;
 	for(k3d::mesh::polyhedra_t::const_iterator polyhedron = Mesh.polyhedra.begin(); polyhedron != Mesh.polyhedra.end(); polyhedron++)
-		for(k3d::polyhedron::edges_t::const_iterator edge = (*polyhedron)->edges.begin(); edge != (*polyhedron)->edges.end(); edge++)
-			edges.insert(detail::ordered_edge_t(*edge));
+		for(k3d::polyhedron::faces_t::const_iterator face = (*polyhedron)->faces.begin(); face != (*polyhedron)->faces.end(); face++)
+		{
+			k3d::split_edge* first_edge = (*face)->first_edge;
+			k3d::split_edge* edge = first_edge;
+			do
+			{
+				edges.insert(detail::ordered_edge_t(edge));
+
+				edge = edge->face_clockwise;
+			}
+			while(edge != first_edge);
+		}
 
 	// Create a mapping of edges-to-one-based-indices ...
 	std::map<detail::ordered_edge_t, unsigned long> edge_map;
@@ -323,10 +331,10 @@ GtsSurface* gts_surface(const k3d::mesh& Mesh)
 			current_edge = current_edge->face_clockwise;
 			if(current_edge)
 				if(current_edge->face_clockwise == first)
-					{
-						triangle.push_back(detail::ordered_edge_t(current_edge));
-						triangle_collection.push_back(triangle);
-					}
+				{
+					triangle.push_back(detail::ordered_edge_t(current_edge));
+					triangle_collection.push_back(triangle);
+				}
 		}
 
 	// Create new GTS surface
@@ -337,46 +345,46 @@ GtsSurface* gts_surface(const k3d::mesh& Mesh)
 	GtsVertex** gts_vertices = static_cast<GtsVertex**>(g_malloc((Mesh.points.size() + 1) * sizeof(GtsVertex*)));
 	guint n = 0;
 	for(k3d::mesh::points_t::const_iterator point = Mesh.points.begin(); point != p_end; ++point)
-		{
-			// Transform coords back to GTS convention
-			const k3d::vector3 coords = (*point)->position;
-			GtsObject* new_vertex = gts_object_new (GTS_OBJECT_CLASS (surface->vertex_class));
-			GtsVertex* v = GTS_VERTEX (new_vertex);
+	{
+		// Transform coords back to GTS convention
+		const k3d::vector3 coords = (*point)->position;
+		GtsObject* new_vertex = gts_object_new (GTS_OBJECT_CLASS (surface->vertex_class));
+		GtsVertex* v = GTS_VERTEX (new_vertex);
 
-			v->p.x = -coords[0];
-			v->p.y = -coords[2];
-			v->p.z = coords[1];
+		v->p.x = -coords[0];
+		v->p.y = -coords[2];
+		v->p.z = coords[1];
 
-			gts_vertices[n++] = v;
-		}
+		gts_vertices[n++] = v;
+	}
 
 	// Add edges
 	GtsEdge** gts_edges = static_cast<GtsEdge**>(g_malloc((edges.size() + 1) * sizeof(GtsEdge*)));
 	n = 0;
 	for(detail::ordered_edges_t::iterator edge = edges.begin(); edge != edges.end(); ++edge)
-		{
-			guint p1 = point_map[edge->first];
-			guint p2 = point_map[edge->second];
-			GtsEdge* new_edge = gts_edge_new (surface->edge_class, gts_vertices[p1 - 1], gts_vertices[p2 - 1]);
+	{
+		guint p1 = point_map[edge->first];
+		guint p2 = point_map[edge->second];
+		GtsEdge* new_edge = gts_edge_new (surface->edge_class, gts_vertices[p1 - 1], gts_vertices[p2 - 1]);
 
-			gts_edges[n++] = new_edge;
-		}
+		gts_edges[n++] = new_edge;
+	}
 
 	// Add triangles
 	for(detail::edge_triangles_t::iterator triangle = triangle_collection.begin(); triangle != triangle_collection.end(); ++triangle)
-		{
-			assert_warning(triangle->size() == 3);
+	{
+		assert_warning(triangle->size() == 3);
 
-			guint s1 = edge_map[(*triangle)[0]];
-			guint s2 = edge_map[(*triangle)[1]];
-			guint s3 = edge_map[(*triangle)[2]];
+		guint s1 = edge_map[(*triangle)[0]];
+		guint s2 = edge_map[(*triangle)[1]];
+		guint s3 = edge_map[(*triangle)[2]];
 
-			// Invert face (because of coordinate system switch)
-			GtsFace* new_face = gts_face_new (surface->face_class,
-				gts_edges[s1 - 1], gts_edges[s3 - 1], gts_edges[s2 - 1]);
+		// Invert face (because of coordinate system switch)
+		GtsFace* new_face = gts_face_new (surface->face_class,
+			gts_edges[s1 - 1], gts_edges[s3 - 1], gts_edges[s2 - 1]);
 
-			gts_surface_add_face (surface, new_face);
-		}
+		gts_surface_add_face (surface, new_face);
+	}
 
 	g_free (gts_vertices);
 	g_free (gts_edges);
@@ -421,24 +429,24 @@ static void myvertex_init (GtsMyVertex * myvertex)
 
 // gts_myvertex_class:
 // define internal function to create myvertex data class following standard
-GtsMyVertexClass * gts_myvertex_class (void)
+GtsMyVertexClass* gts_myvertex_class(void)
 {
-	static GtsMyVertexClass * klass = 0;
+	static GtsMyVertexClass* klass = 0;
 
-	if (klass == 0)
-		{
-			GtsObjectClassInfo myvertex_info = {
-				"GtsMyVertex",
-				sizeof (GtsMyVertex),
-				sizeof (GtsMyVertexClass),
-				(GtsObjectClassInitFunc) 0,
-				(GtsObjectInitFunc) myvertex_init,
-				(GtsArgSetFunc) 0,
-				(GtsArgGetFunc) 0
-				};
+	if(klass == 0)
+	{
+		GtsObjectClassInfo myvertex_info = {
+			"GtsMyVertex",
+			sizeof (GtsMyVertex),
+			sizeof (GtsMyVertexClass),
+			(GtsObjectClassInitFunc) 0,
+			(GtsObjectInitFunc) myvertex_init,
+			(GtsArgSetFunc) 0,
+			(GtsArgGetFunc) 0
+			};
 
-			klass = static_cast<GtsMyVertexClass*>(gts_object_class_new (GTS_OBJECT_CLASS (gts_vertex_class ()), &myvertex_info));
-		}
+		klass = static_cast<GtsMyVertexClass*>(gts_object_class_new (GTS_OBJECT_CLASS (gts_vertex_class ()), &myvertex_info));
+	}
 
 	return klass;
 }
@@ -467,7 +475,7 @@ typedef struct _GtsMyEdge          GtsMyEdge;
 typedef struct _GtsMyEdgeClass     GtsMyEdgeClass;
 
 // Prototype for myedge data class function following standard
-GtsMyEdgeClass * gts_myedge_class (void);
+GtsMyEdgeClass* gts_myedge_class(void);
 
 // Define an init function for myedge data class
 static void myedge_init (GtsMyEdge * myedge)
@@ -477,24 +485,24 @@ static void myedge_init (GtsMyEdge * myedge)
 
 // gts_myedge_class:
 // define internal function to create myedge data class following standard
-GtsMyEdgeClass * gts_myedge_class (void)
+GtsMyEdgeClass* gts_myedge_class(void)
 {
-	static GtsMyEdgeClass * klass = 0;
+	static GtsMyEdgeClass* klass = 0;
 
 	if (klass == 0)
-		{
-			GtsObjectClassInfo myedge_info = {
-				"GtsMyEdge",
-				sizeof (GtsMyEdge),
-				sizeof (GtsMyEdgeClass),
-				(GtsObjectClassInitFunc) 0,
-				(GtsObjectInitFunc) myedge_init,
-				(GtsArgSetFunc) 0,
-				(GtsArgGetFunc) 0
-				};
+	{
+		GtsObjectClassInfo myedge_info = {
+			"GtsMyEdge",
+			sizeof (GtsMyEdge),
+			sizeof (GtsMyEdgeClass),
+			(GtsObjectClassInitFunc) 0,
+			(GtsObjectInitFunc) myedge_init,
+			(GtsArgSetFunc) 0,
+			(GtsArgGetFunc) 0
+			};
 
-			klass = static_cast<GtsMyEdgeClass*>(gts_object_class_new (GTS_OBJECT_CLASS (gts_edge_class ()), &myedge_info));
-		}
+		klass = static_cast<GtsMyEdgeClass*>(gts_object_class_new (GTS_OBJECT_CLASS (gts_edge_class ()), &myedge_info));
+	}
 
 	return klass;
 }
@@ -539,24 +547,24 @@ static void myface_init (GtsMyFace * myface)
 
 // gts_myface_class:
 // define internal function to create myface data class following standard
-GtsMyFaceClass * gts_myface_class (void)
+GtsMyFaceClass* gts_myface_class (void)
 {
-	static GtsMyFaceClass * klass = 0;
+	static GtsMyFaceClass* klass = 0;
 
-	if (klass == 0)
-		{
-			GtsObjectClassInfo myface_info = {
-				"GtsMyFace",
-				sizeof (GtsMyFace),
-				sizeof (GtsMyFaceClass),
-				(GtsObjectClassInitFunc) 0,
-				(GtsObjectInitFunc) myface_init,
-				(GtsArgSetFunc) 0,
-				(GtsArgGetFunc) 0
-				};
+	if(klass == 0)
+	{
+		GtsObjectClassInfo myface_info = {
+			"GtsMyFace",
+			sizeof (GtsMyFace),
+			sizeof (GtsMyFaceClass),
+			(GtsObjectClassInitFunc) 0,
+			(GtsObjectInitFunc) myface_init,
+			(GtsArgSetFunc) 0,
+			(GtsArgGetFunc) 0
+			};
 
-			klass = static_cast<GtsMyFaceClass*>(gts_object_class_new (GTS_OBJECT_CLASS (gts_face_class ()), &myface_info));
-		}
+		klass = static_cast<GtsMyFaceClass*>(gts_object_class_new (GTS_OBJECT_CLASS (gts_face_class ()), &myface_info));
+	}
 
 	return klass;
 }
@@ -576,61 +584,58 @@ GtsSurface* gts_polygonal_surface(const k3d::mesh& Mesh, unsigned long& Polyhedr
 	std::vector<unsigned long> triangles_sizes;
 	unsigned long polygon_number = 0;
 	for(k3d::mesh::polyhedra_t::const_iterator polyhedron_iterator = Mesh.polyhedra.begin(); polyhedron_iterator != Mesh.polyhedra.end(); ++polyhedron_iterator)
+	{
+		k3d::polyhedron& polyhedron = **polyhedron_iterator;
+
+		// Process each face ...
+		for(k3d::polyhedron::faces_t::const_iterator face = polyhedron.faces.begin(); face != polyhedron.faces.end(); face++)
 		{
-			k3d::polyhedron& polyhedron = **polyhedron_iterator;
+			k3d::polyhedron::faces_t faces;
+			faces.push_back(*face);
 
-			// Process each face ...
-			for(k3d::polyhedron::faces_t::const_iterator face = polyhedron.faces.begin(); face != polyhedron.faces.end(); face++)
+			k3d::split_edge* current_edge = (*face)->first_edge;
+			while(current_edge && current_edge != (*face)->first_edge)
 			{
-				k3d::polyhedron::faces_t faces;
-				faces.push_back(*face);
+				current_edge = current_edge->face_clockwise;
+			}
 
-				k3d::polyhedron::edges_t edges;
+			// Triangulate ...
+			k3d::polyhedron::faces_t new_faces;
+			k3d::triangulate(faces, new_faces, new_points);
+
+			// For each face ...
+			for(k3d::polyhedron::faces_t::const_iterator face = new_faces.begin(); face != new_faces.end(); face++)
+			{
+				detail::edge_triangle_t triangle;
 				k3d::split_edge* current_edge = (*face)->first_edge;
-				while(current_edge && current_edge != (*face)->first_edge)
-				{
-					edges.push_back(current_edge);
-					current_edge = current_edge->face_clockwise;
-				}
+				assert_warning(current_edge);
+				triangle.push_back(detail::ordered_edge_t(current_edge));
 
-				// Triangulate ...
-				k3d::polyhedron::faces_t new_faces;
-				k3d::polyhedron::edges_t new_edges;
-				k3d::triangulate(faces.begin(), faces.end(), std::back_inserter(new_faces), std::back_inserter(new_edges), std::back_inserter(new_points));
+				current_edge = current_edge->face_clockwise;
+				assert_warning(current_edge);
+				triangle.push_back(detail::ordered_edge_t(current_edge));
 
-				// For each face ...
-				for(k3d::polyhedron::faces_t::const_iterator face = new_faces.begin(); face != new_faces.end(); face++)
-				{
-					detail::edge_triangle_t triangle;
-					k3d::split_edge* current_edge = (*face)->first_edge;
-					assert_warning(current_edge);
-					triangle.push_back(detail::ordered_edge_t(current_edge));
+				current_edge = current_edge->face_clockwise;
+				assert_warning(current_edge && current_edge->face_clockwise == (*face)->first_edge);
+				triangle.push_back(detail::ordered_edge_t(current_edge));
 
-					current_edge = current_edge->face_clockwise;
-					assert_warning(current_edge);
-					triangle.push_back(detail::ordered_edge_t(current_edge));
+				triangle_collection.push_back(triangle);
+				triangles_polyhedron.push_back(PolyhedronIndex);
+				triangles_polygon.push_back(polygon_number);
+				triangles_sizes.push_back(new_faces.size());
 
-					current_edge = current_edge->face_clockwise;
-					assert_warning(current_edge && current_edge->face_clockwise == (*face)->first_edge);
-					triangle.push_back(detail::ordered_edge_t(current_edge));
-
-					triangle_collection.push_back(triangle);
-					triangles_polyhedron.push_back(PolyhedronIndex);
-					triangles_polygon.push_back(polygon_number);
-					triangles_sizes.push_back(new_faces.size());
-
-					// Save edges for mapping
-					std::copy(triangle.begin(), triangle.end(), std::inserter(edge_collection, edge_collection.end()));
-				}
-
-				// Update counter
-				polygon_number++;
+				// Save edges for mapping
+				std::copy(triangle.begin(), triangle.end(), std::inserter(edge_collection, edge_collection.end()));
 			}
 
 			// Update counter
-			PolyhedronIndex++;
+			polygon_number++;
 		}
 
+		// Update counter
+		PolyhedronIndex++;
+	}
+
 	// Create new GTS surface
 	GtsSurface* surface = gts_surface_new(gts_surface_class(), GTS_FACE_CLASS (detail::gts_myface_class()), GTS_EDGE_CLASS (detail::gts_myedge_class()), GTS_VERTEX_CLASS (detail::gts_myvertex_class()));
 	return_val_if_fail(surface, 0);
@@ -642,23 +647,23 @@ GtsSurface* gts_polygonal_surface(const k3d::mesh& Mesh, unsigned long& Polyhedr
 	gulong vertex_number = 1;
 	k3d::mesh::points_t::const_iterator p_end(new_points.end());
 	for(k3d::mesh::points_t::const_iterator point = new_points.begin(); point != p_end; ++point)
-		{
-			// Transform coords back to GTS convention
-			const k3d::vector3 coords = (*point)->position;
-			GtsObject* new_vertex = gts_object_new (GTS_OBJECT_CLASS (surface->vertex_class));
-			detail::GtsMyVertex* myvertex = GTS_MYVERTEX(new_vertex);
-			myvertex->number = vertex_number++;
-			GtsVertex* v = GTS_VERTEX (new_vertex);
+	{
+		// Transform coords back to GTS convention
+		const k3d::vector3 coords = (*point)->position;
+		GtsObject* new_vertex = gts_object_new (GTS_OBJECT_CLASS (surface->vertex_class));
+		detail::GtsMyVertex* myvertex = GTS_MYVERTEX(new_vertex);
+		myvertex->number = vertex_number++;
+		GtsVertex* v = GTS_VERTEX (new_vertex);
 
-			v->p.x = -coords[0];
-			v->p.y = -coords[2];
-			v->p.z = coords[1];
+		v->p.x = -coords[0];
+		v->p.y = -coords[2];
+		v->p.z = coords[1];
 
-			gts_vertices[n++] = v;
+		gts_vertices[n++] = v;
 
-			// Create a mapping of points-to-one-based-indices ...
-			point_map[*point] = n;
-		}
+		// Create a mapping of points-to-one-based-indices ...
+		point_map[*point] = n;
+	}
 
 	// Create a mapping of edges-to-one-based-indices ...
 	std::map<detail::ordered_edge_t, unsigned long> edge_map;
@@ -670,40 +675,40 @@ GtsSurface* gts_polygonal_surface(const k3d::mesh& Mesh, unsigned long& Polyhedr
 	n = 0;
 	gulong edge_number = 1;
 	for(detail::ordered_edges_t::iterator edge = edge_collection.begin(); edge != edge_collection.end(); ++edge)
-		{
-			guint p1 = point_map[edge->first];
-			guint p2 = point_map[edge->second];
-			GtsEdge* new_edge = gts_edge_new (surface->edge_class, gts_vertices[p1 - 1], gts_vertices[p2 - 1]);
+	{
+		guint p1 = point_map[edge->first];
+		guint p2 = point_map[edge->second];
+		GtsEdge* new_edge = gts_edge_new (surface->edge_class, gts_vertices[p1 - 1], gts_vertices[p2 - 1]);
 
-			gts_edges[n++] = new_edge;
+		gts_edges[n++] = new_edge;
 
-			detail::GtsMyEdge* myedge = GTS_MYEDGE(new_edge);
-			myedge->number = edge_number++;
-		}
+		detail::GtsMyEdge* myedge = GTS_MYEDGE(new_edge);
+		myedge->number = edge_number++;
+	}
 
 	// Add triangles
 	gulong face_number = 1;
 	unsigned long index = 0;
 	for(detail::edge_triangles_t::iterator triangle = triangle_collection.begin(); triangle != triangle_collection.end(); ++triangle)
-		{
-			assert_warning(triangle->size() == 3);
+	{
+		assert_warning(triangle->size() == 3);
 
-			guint s1 = edge_map[(*triangle)[0]];
-			guint s2 = edge_map[(*triangle)[1]];
-			guint s3 = edge_map[(*triangle)[2]];
+		guint s1 = edge_map[(*triangle)[0]];
+		guint s2 = edge_map[(*triangle)[1]];
+		guint s3 = edge_map[(*triangle)[2]];
 
-			// Invert face (because of coordinate system switch)
-			GtsFace* new_face = gts_face_new (surface->face_class,
-				gts_edges[s1 - 1], gts_edges[s3 - 1], gts_edges[s2 - 1]);
+		// Invert face (because of coordinate system switch)
+		GtsFace* new_face = gts_face_new (surface->face_class,
+			gts_edges[s1 - 1], gts_edges[s3 - 1], gts_edges[s2 - 1]);
 
-			detail::GtsMyFace* myface = GTS_MYFACE(new_face);
-			myface->number = face_number++;
-			myface->polyhedron_number = triangles_polyhedron[index];
-			myface->polygon_number = triangles_polygon[index];
-			myface->polygon_size = triangles_sizes[index];
+		detail::GtsMyFace* myface = GTS_MYFACE(new_face);
+		myface->number = face_number++;
+		myface->polyhedron_number = triangles_polyhedron[index];
+		myface->polygon_number = triangles_polygon[index];
+		myface->polygon_size = triangles_sizes[index];
 
-			gts_surface_add_face (surface, new_face);
-		}
+		gts_surface_add_face (surface, new_face);
+	}
 
 	g_free (gts_vertices);
 	g_free (gts_edges);
@@ -827,30 +832,30 @@ bool copy_polygonal_surface(GtsSurface* Surface, const k3d::mesh& OldMesh, k3d::
 		// Determine order of vertices
 		unsigned long p[3];
 		if(edge1_first == edge2_first)
-			{
-				p[0] = edge1_second;
-				p[1] = edge1_first;
-				p[2] = edge2_second;
-			}
+		{
+			p[0] = edge1_second;
+			p[1] = edge1_first;
+			p[2] = edge2_second;
+		}
 		else if(edge1_first == edge2_second)
-			{
-				p[0] = edge1_second;
-				p[1] = edge1_first;
-				p[2] = edge2_first;
-			}
+		{
+			p[0] = edge1_second;
+			p[1] = edge1_first;
+			p[2] = edge2_first;
+		}
 		else if(edge1_second == edge2_first)
-			{
-				p[0] = edge1_first;
-				p[1] = edge1_second;
-				p[2] = edge2_second;
-			}
+		{
+			p[0] = edge1_first;
+			p[1] = edge1_second;
+			p[2] = edge2_second;
+		}
 		else
-			// edge1_second == edge2_second
-			{
-				p[0] = edge1_first;
-				p[1] = edge1_second;
-				p[2] = edge2_first;
-			}
+		// edge1_second == edge2_second
+		{
+			p[0] = edge1_first;
+			p[1] = edge1_second;
+			p[2] = edge2_first;
+		}
 
 		// Create a triangle ...
 		k3d::split_edge* edge1 = new k3d::split_edge(detail::points[p[0]]);
@@ -862,11 +867,7 @@ bool copy_polygonal_surface(GtsSurface* Surface, const k3d::mesh& OldMesh, k3d::
 		edge3->face_clockwise = edge2;
 		edge2->face_clockwise = edge1;
 
-		polyhedron.edges.push_back(edge1);
-		polyhedron.edges.push_back(edge2);
-		polyhedron.edges.push_back(edge3);
-
-		k3d::face* const face = new k3d::face(edge1);
+		k3d::face* const face = new k3d::face(edge1, 0);
 		return_val_if_fail(face, false);
 
 		polyhedron.faces.push_back(face);
@@ -875,9 +876,10 @@ bool copy_polygonal_surface(GtsSurface* Surface, const k3d::mesh& OldMesh, k3d::
 	return_val_if_fail(k3d::is_valid(polyhedron), false);
 	//assert_warning(k3d::is_solid(polyhedron));
 
+	k3d::set_companions(polyhedron);
+
 	return true;
 }
 
 } // namespace libk3dgts
 
-
