@@ -1,7 +1,7 @@
 // K-3D
-// Copyright (c) 1995-2004, Timothy M. Shead
+// Copyright (c) 2004-2005, Romain Behar
 //
-// Contact: tshead@k-3d.com
+// Contact: romainbehar@yahoo.com
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public
@@ -21,25 +21,23 @@
 		\author Romain Behar (romainbehar@yahoo.com)
 */
 
-#include <k3dsdk/classes.h>
 #include <k3dsdk/color.h>
 #include <k3dsdk/file_helpers.h>
+#include <k3dsdk/i18n.h>
 #include <k3dsdk/imaterial.h>
 #include <k3dsdk/material.h>
-#include <k3dsdk/material_collection.h>
+#include <k3dsdk/material_client.h>
 #include <k3dsdk/measurement.h>
 #include <k3dsdk/mesh_source.h>
 #include <k3dsdk/module.h>
-#include <k3dsdk/object.h>
-#include <k3dsdk/path_data.h>
-#include <k3dsdk/persistence.h>
-#include <k3dsdk/plugins.h>
+#include <k3dsdk/node.h>
+#include <k3dsdk/persistent.h>
 #include <k3dsdk/property.h>
 #include <k3dsdk/string_modifiers.h>
-#include <k3dsdk/transform.h>
 
-#include <boost/filesystem/fstream.hpp>
+#include <k3dsdk/fstream.h>
 #include <boost/filesystem/path.hpp>
+#include <boost/filesystem/operations.hpp>
 
 #include <cmath>
 #include <fstream>
@@ -147,7 +145,7 @@ typedef std::vector<polygon> polygons_t;
 polygons_t polygons;
 
 // Bounding-box
-k3d::bounding_box bounding_box;
+k3d::bounding_box3 bounding_box;
 double bbox_x = 0;
 double bbox_y = 0;
 double bbox_z = 0;
@@ -173,7 +171,7 @@ k3d::color colors[color_number] = {
 };
 
 // Save an object from store to disc
-bool add_geometry(unsigned long color, k3d::mesh* mesh)
+bool add_geometry(unsigned long color, k3d::mesh* mesh, k3d::imaterial* const Material)
 {
 	std::vector<k3d::point*> points;
 
@@ -192,7 +190,7 @@ bool add_geometry(unsigned long color, k3d::mesh* mesh)
 
 	for(unsigned long t = 0; t < polygons.size(); t++)
 		{
-			k3d::polyhedron::edges_t edges;
+			std::vector<k3d::split_edge*> edges;
 			edges.push_back(new k3d::split_edge(points[polygons[t].a]));
 			edges.push_back(new k3d::split_edge(points[polygons[t].b]));
 			edges.push_back(new k3d::split_edge(points[polygons[t].c]));
@@ -202,9 +200,8 @@ bool add_geometry(unsigned long color, k3d::mesh* mesh)
 
 			k3d::loop_edges(edges.begin(), edges.end());
 			k3d::polyhedron& polyhedron = *mesh->polyhedra.back();
-			polyhedron.edges.insert(polyhedron.edges.end(), edges.begin(), edges.end());
 
-			k3d::face* new_face = new k3d::face(edges.front());
+			k3d::face* new_face = new k3d::face(edges.front(), Material);
 			return_val_if_fail(new_face, false);
 			polyhedron.faces.push_back(new_face);
 
@@ -221,7 +218,7 @@ bool add_geometry(unsigned long color, k3d::mesh* mesh)
 
 
 // Here we build a cube shape directly on the input vectors
-void add_cube(k3d::vector3 start, k3d::vector3 end, k3d::vector3 up, unsigned long color, k3d::mesh* mesh)
+void add_cube(k3d::vector3 start, k3d::vector3 end, k3d::vector3 up, unsigned long color, k3d::mesh* mesh, k3d::imaterial* const Material)
 {
 	// Check size
 	k3d::vector3 direction = end - start;
@@ -270,13 +267,13 @@ void add_cube(k3d::vector3 start, k3d::vector3 end, k3d::vector3 up, unsigned lo
 	polygons.push_back(polygon(0, 1, 2, 3));
 	polygons.push_back(polygon(7, 6, 5, 4));
 
-	add_geometry(color, mesh);
+	add_geometry(color, mesh, Material);
 }
 
 // The lastxxx vars are used to store the previous top of the cylinder
 // for connecting a next one; since the vars are stacked for [] we can
 // connect correctly according to current nesting level
-void add_cylinder(k3d::vector3 start, k3d::vector3 end, k3d::vector3 up, unsigned long color, k3d::mesh* mesh)
+void add_cylinder(k3d::vector3 start, k3d::vector3 end, k3d::vector3 up, unsigned long color, k3d::mesh* mesh, k3d::imaterial* const Material)
 {
 	// Check size
 	k3d::vector3 direction = end - start;
@@ -361,7 +358,7 @@ void add_cylinder(k3d::vector3 start, k3d::vector3 end, k3d::vector3 up, unsigne
 	polygons.push_back(polygon(6, 14, 15, 7));
 	polygons.push_back(polygon(7, 15, 8, 0));
 
-	add_geometry(color, mesh);
+	add_geometry(color, mesh, Material);
 
 	// Save cylinder's parameters and top vertices
 	last_col = color;
@@ -399,10 +396,10 @@ bool ls_line(std::istream& file, std::string& linebuffer)
 bool load_configuration_values(const boost::filesystem::path& file_path, double& recursion, double& basic_angle, double& thickness)
 {
 	// Open configuration file
-	boost::filesystem::ifstream file(file_path);
+	k3d::filesystem::ifstream file(file_path);
 	if(!file.good())
 		{
-			std::cerr << __PRETTY_FUNCTION__ << ": error opening [" << file_path.native_file_string() << "]" << std::endl;
+			k3d::log() << error << k3d_file_reference << ": error opening [" << file_path.native_file_string() << "]" << std::endl;
 			return 0.0;
 		}
 
@@ -429,10 +426,10 @@ bool load_configuration_values(const boost::filesystem::path& file_path, double&
 bool load_configuration_rules(const double recursion, const double basic_angle, const double thickness, const boost::filesystem::path& file_path)
 {
 	// Open grammar file
-	boost::filesystem::ifstream file(file_path);
+	k3d::filesystem::ifstream file(file_path);
 	if(!file.good())
 		{
-			std::cerr << __PRETTY_FUNCTION__ << ": error opening [" << file_path.native_file_string() << "]" << std::endl;
+			k3d::log() << error << k3d_file_reference << ": error opening [" << file_path.native_file_string() << "]" << std::endl;
 			return false;
 		}
 
@@ -875,7 +872,7 @@ k3d::vector3 rotate(const k3d::vector3& In)
 }
 
 // Process a production string and generate form
-void L_draw(k3d::mesh* mesh)
+void L_draw(k3d::mesh* mesh, k3d::imaterial* const Material)
 {
 	// Save values
 	double thick_l = 0;
@@ -1272,9 +1269,9 @@ void L_draw(k3d::mesh* mesh)
 
 						k3d::vector3 end = pos + dis2 * fow;
 						if(closed_form)
-							add_cylinder(pos, end, upp, col, mesh);
+							add_cylinder(pos, end, upp, col, mesh, Material);
 						else
-							add_cube(pos, end, upp, col, mesh);
+							add_cube(pos, end, upp, col, mesh, Material);
 
 						pos = end;
 						dis2 = save.dis2;
@@ -1293,9 +1290,9 @@ void L_draw(k3d::mesh* mesh)
 
 						k3d::vector3 end = pos + dis * fow;
 						if(closed_form)
-							add_cylinder(pos, end, upp, col, mesh);
+							add_cylinder(pos, end, upp, col, mesh, Material);
 						else
-							add_cube(pos, end, upp, col, mesh);
+							add_cube(pos, end, upp, col, mesh, Material);
 
 						pos = end;
 						dis = save.dis;
@@ -1428,7 +1425,7 @@ void L_draw(k3d::mesh* mesh)
 								for(unsigned long j = 1; j < vertices.size() - 1; j++)
 									polygons.push_back(polygon(0, j, j + 1, j + 1));
 
-								add_geometry(col, mesh);
+								add_geometry(col, mesh, Material);
 							}
 
 						poly_on = false;
@@ -1456,7 +1453,7 @@ void L_draw(k3d::mesh* mesh)
 		}
 }
 
-bool l_parser(const unsigned long random_seed, const bool closedform, const double trope, const unsigned long mutations, const unsigned long mutation_seed, const unsigned long maximal_stack_size, k3d::mesh* mesh)
+bool l_parser(const unsigned long random_seed, const bool closedform, const double trope, const unsigned long mutations, const unsigned long mutation_seed, const unsigned long maximal_stack_size, k3d::mesh* mesh, k3d::imaterial* const Material)
 {
 	// Make sure we have a polyhedron to fill
 	return_val_if_fail(mesh->polyhedra.size(), false);
@@ -1482,7 +1479,7 @@ bool l_parser(const unsigned long random_seed, const bool closedform, const doub
 	// Parse production string and create geometry
 	srand(random_seed);
 	max_stack_size = maximal_stack_size;
-	L_draw(mesh);
+	L_draw(mesh, Material);
 
 	return true;
 }
@@ -1490,63 +1487,41 @@ bool l_parser(const unsigned long random_seed, const bool closedform, const doub
 } // namespace lparser
 
 /////////////////////////////////////////////////////////////////////////////
-// l_parser_implementation
+// l_parser
 
-class l_parser_implementation :
-	public k3d::material_collection<k3d::mesh_source<k3d::persistent<k3d::object> > >
+class l_parser :
+	public k3d::material_client<k3d::mesh_source<k3d::persistent<k3d::node> > >
 {
-	typedef k3d::material_collection<k3d::mesh_source<k3d::persistent<k3d::object> > > base;
+	typedef k3d::material_client<k3d::mesh_source<k3d::persistent<k3d::node> > > base;
 
 public:
-	l_parser_implementation(k3d::idocument& Document) :
+	l_parser(k3d::idocument& Document) :
 		base(Document),
-		m_file_path(k3d::init_name("file") + k3d::init_description("File path [string]") + k3d::init_value(boost::filesystem::path(k3d::application().share_path() / "lsystem/abop-nested-polygon-leaves.ls")) + k3d::init_document(Document)),
-		m_recursion(k3d::init_name("growth") + k3d::init_description("Growth value [number]") + k3d::init_value(5.0) + k3d::init_precision(2) + k3d::init_step_increment(0.1) + k3d::init_constraint(k3d::constraint::minimum(1.0)) + k3d::init_units(typeid(k3d::measurement::scalar)) + k3d::init_document(Document)),
-		m_basic_angle(k3d::init_name("basic_angle") + k3d::init_description("Basic Angle [number]") + k3d::init_value(10.0) + k3d::init_precision(2) + k3d::init_step_increment(1.0) + k3d::init_units(typeid(k3d::measurement::scalar)) + k3d::init_document(Document)),
-		m_thickness(k3d::init_name("thickness") + k3d::init_description("Thickness [number]") + k3d::init_value(50.0) + k3d::init_precision(2) + k3d::init_step_increment(1.0) + k3d::init_units(typeid(k3d::measurement::scalar)) + k3d::init_document(Document)),
-		m_random_seed(k3d::init_name("random_seed") + k3d::init_description("Random Seed [number]") + k3d::init_value(0) + k3d::init_precision(0) + k3d::init_step_increment(1) + k3d::init_units(typeid(k3d::measurement::scalar)) + k3d::init_document(Document)),
-		m_size(k3d::init_name("size") + k3d::init_description("Size [number]") + k3d::init_value(5.0) + k3d::init_precision(2) + k3d::init_step_increment(0.1) + k3d::init_units(typeid(k3d::measurement::scalar)) + k3d::init_document(Document)),
-		m_closed_form(k3d::init_name("closed_form") + k3d::init_description("Closed form [boolean]") + k3d::init_value(true) + k3d::init_document(Document)),
-		m_mutations(k3d::init_name("mutations") + k3d::init_description("Mutations [number]") + k3d::init_value(0) + k3d::init_precision(0) + k3d::init_step_increment(1) + k3d::init_units(typeid(k3d::measurement::scalar)) + k3d::init_document(Document)),
-		m_mutation_seed(k3d::init_name("mutation_seed") + k3d::init_description("Mutation Seed [number]") + k3d::init_value(0) + k3d::init_precision(0) + k3d::init_step_increment(1) + k3d::init_units(typeid(k3d::measurement::scalar)) + k3d::init_document(Document)),
-		m_max_stack_size(k3d::init_name("max_stack_size") + k3d::init_description("Max Stack Size [number]") + k3d::init_value(1000) + k3d::init_precision(0) + k3d::init_step_increment(1) + k3d::init_units(typeid(k3d::measurement::scalar)) + k3d::init_document(Document))
+		m_file_path(init_owner(*this) + init_name("file") + init_label(_("File")) + init_description(_("File path for lsystem description")) + init_value(boost::filesystem::path(k3d::application().share_path() / "lsystem/abop-nested-polygon-leaves.ls")) + init_path_mode(k3d::ipath_property::READ) + init_path_type("lsystems")),
+		m_recursion(init_owner(*this) + init_name("growth") + init_label(_("Growth")) + init_description(_("Growth value")) + init_value(5.0) + init_precision(2) + init_step_increment(0.1) + init_constraint(constraint::minimum(1.0)) + init_units(typeid(k3d::measurement::scalar))),
+		m_basic_angle(init_owner(*this) + init_name("basic_angle") + init_label(_("Basic angle")) + init_description(_("Basic angle")) + init_value(10.0) + init_precision(2) + init_step_increment(1.0) + init_units(typeid(k3d::measurement::scalar))),
+		m_thickness(init_owner(*this) + init_name("thickness") + init_label(_("Thickness")) + init_description(_("Thickness")) + init_value(50.0) + init_precision(2) + init_step_increment(1.0) + init_units(typeid(k3d::measurement::scalar))),
+		m_random_seed(init_owner(*this) + init_name("random_seed") + init_label(_("Random seed")) + init_description(_("Random seed")) + init_value(0) + init_precision(0) + init_step_increment(1) + init_units(typeid(k3d::measurement::scalar))),
+		m_size(init_owner(*this) + init_name("size") + init_label(_("Size")) + init_description(_("Size")) + init_value(5.0) + init_precision(2) + init_step_increment(0.1) + init_units(typeid(k3d::measurement::scalar))),
+		m_closed_form(init_owner(*this) + init_name("closed_form") + init_label(_("Closed form")) + init_description(_("Closed form")) + init_value(true)),
+		m_mutations(init_owner(*this) + init_name("mutations") + init_label(_("Mutations")) + init_description(_("Mutations")) + init_value(0) + init_precision(0) + init_step_increment(1) + init_units(typeid(k3d::measurement::scalar))),
+		m_mutation_seed(init_owner(*this) + init_name("mutation_seed") + init_label(_("Mutation seed")) + init_description(_("Mutation seed")) + init_value(0) + init_precision(0) + init_step_increment(1) + init_units(typeid(k3d::measurement::scalar))),
+		m_max_stack_size(init_owner(*this) + init_name("max_stack_size") + init_label(_("Max stack size")) + init_description(_("Max stack size")) + init_value(1000) + init_precision(0) + init_step_increment(1) + init_units(typeid(k3d::measurement::scalar)))
 	{
-		enable_serialization(k3d::persistence::proxy(m_file_path));
-		enable_serialization(k3d::persistence::proxy(m_recursion));
-		enable_serialization(k3d::persistence::proxy(m_basic_angle));
-		enable_serialization(k3d::persistence::proxy(m_thickness));
-		enable_serialization(k3d::persistence::proxy(m_random_seed));
-		enable_serialization(k3d::persistence::proxy(m_size));
-		enable_serialization(k3d::persistence::proxy(m_closed_form));
-		enable_serialization(k3d::persistence::proxy(m_mutations));
-		enable_serialization(k3d::persistence::proxy(m_mutation_seed));
-		enable_serialization(k3d::persistence::proxy(m_max_stack_size));
+		m_material.changed_signal().connect(sigc::mem_fun(*this, &l_parser::on_reset_geometry));
 
-		register_property(m_file_path);
-		register_property(m_recursion);
-		register_property(m_basic_angle);
-		register_property(m_thickness);
-		register_property(m_random_seed);
-		register_property(m_size);
-		register_property(m_closed_form);
-		register_property(m_mutations);
-		register_property(m_mutation_seed);
-		register_property(m_max_stack_size);
+		m_file_path.changed_signal().connect(sigc::mem_fun(*this, &l_parser::on_new_file));
+		m_recursion.changed_signal().connect(sigc::mem_fun(*this, &l_parser::on_reset_geometry));
+		m_basic_angle.changed_signal().connect(sigc::mem_fun(*this, &l_parser::on_reset_geometry));
+		m_thickness.changed_signal().connect(sigc::mem_fun(*this, &l_parser::on_reset_geometry));
+		m_random_seed.changed_signal().connect(sigc::mem_fun(*this, &l_parser::on_reset_geometry));
+		m_size.changed_signal().connect(sigc::mem_fun(*this, &l_parser::on_reset_geometry));
+		m_closed_form.changed_signal().connect(sigc::mem_fun(*this, &l_parser::on_reset_geometry));
+		m_mutations.changed_signal().connect(sigc::mem_fun(*this, &l_parser::on_reset_geometry));
+		m_mutation_seed.changed_signal().connect(sigc::mem_fun(*this, &l_parser::on_reset_geometry));
+		m_max_stack_size.changed_signal().connect(sigc::mem_fun(*this, &l_parser::on_reset_geometry));
 
-		m_material.changed_signal().connect(SigC::slot(*this, &l_parser_implementation::on_reset_geometry));
-
-		m_file_path.changed_signal().connect(SigC::slot(*this, &l_parser_implementation::on_new_file));
-		m_recursion.changed_signal().connect(SigC::slot(*this, &l_parser_implementation::on_reset_geometry));
-		m_basic_angle.changed_signal().connect(SigC::slot(*this, &l_parser_implementation::on_reset_geometry));
-		m_thickness.changed_signal().connect(SigC::slot(*this, &l_parser_implementation::on_reset_geometry));
-		m_random_seed.changed_signal().connect(SigC::slot(*this, &l_parser_implementation::on_reset_geometry));
-		m_size.changed_signal().connect(SigC::slot(*this, &l_parser_implementation::on_reset_geometry));
-		m_closed_form.changed_signal().connect(SigC::slot(*this, &l_parser_implementation::on_reset_geometry));
-		m_mutations.changed_signal().connect(SigC::slot(*this, &l_parser_implementation::on_reset_geometry));
-		m_mutation_seed.changed_signal().connect(SigC::slot(*this, &l_parser_implementation::on_reset_geometry));
-		m_max_stack_size.changed_signal().connect(SigC::slot(*this, &l_parser_implementation::on_reset_geometry));
-
-		m_output_mesh.need_data_signal().connect(SigC::slot(*this, &l_parser_implementation::on_create_geometry));
+		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &l_parser::on_create_geometry));
 
 		// Init with default example
 		init_lsystem();
@@ -1555,18 +1530,18 @@ public:
 	void init_lsystem()
 	{
 		// Load configuration file
-		const boost::filesystem::path file_path = m_file_path.property_value();
+		const boost::filesystem::path file_path = m_file_path.value();
 		double recursion, basic_angle, thickness;
 		if(!lparser::load_configuration_values(file_path, recursion, basic_angle, thickness))
 			return;
 
 		// Set configuration's default recursion values
-		k3d::set_property_value(m_recursion, recursion);
-		k3d::set_property_value(m_basic_angle, basic_angle);
-		k3d::set_property_value(m_thickness, thickness);
+		k3d::set_value(m_recursion, recursion);
+		k3d::set_value(m_basic_angle, basic_angle);
+		k3d::set_value(m_thickness, thickness);
 
 		// Reset bounding box
-		lparser::bounding_box = k3d::bounding_box();
+		lparser::bounding_box = k3d::bounding_box3();
 		lparser::bbox_x = lparser::bbox_y = lparser::bbox_z = 0;
 	}
 
@@ -1583,18 +1558,19 @@ public:
 
 	k3d::mesh* on_create_geometry()
 	{
-		const double recursion = m_recursion.property_value();
-		const double basic_angle = m_basic_angle.property_value();
-		const double thickness = m_thickness.property_value();
-		const unsigned long random_seed = m_random_seed.property_value();
-		const double size = m_size.property_value();
-		const bool closedform = m_closed_form.property_value();
-		const unsigned long mutations = m_mutations.property_value();
-		const unsigned long mutation_seed = m_mutation_seed.property_value();
-		const unsigned long max_stack_size = m_max_stack_size.property_value();
+		const double recursion = m_recursion.value();
+		const double basic_angle = m_basic_angle.value();
+		const double thickness = m_thickness.value();
+		const unsigned long random_seed = m_random_seed.value();
+		const double size = m_size.value();
+		const bool closedform = m_closed_form.value();
+		const unsigned long mutations = m_mutations.value();
+		const unsigned long mutation_seed = m_mutation_seed.value();
+		const unsigned long max_stack_size = m_max_stack_size.value();
+		k3d::imaterial* const material = m_material.value();
 
 		// Load configuration file
-		const boost::filesystem::path file_path = m_file_path.property_value();
+		const boost::filesystem::path file_path = m_file_path.value();
 		if(!lparser::load_configuration_rules(recursion, basic_angle, thickness, file_path))
 			return 0;
 
@@ -1603,7 +1579,7 @@ public:
 		k3d::polyhedron* polyhedron = new k3d::polyhedron();
 		return_val_if_fail(polyhedron, 0);
 		mesh->polyhedra.push_back(polyhedron);
-		lparser::l_parser(random_seed, closedform, 0, mutations, mutation_seed, max_stack_size, mesh.get());
+		lparser::l_parser(random_seed, closedform, 0, mutations, mutation_seed, max_stack_size, mesh.get(), material);
 
 		// Cache first bounding box to allow growth
 		if(lparser::bbox_x == 0 && lparser::bbox_y == 0 && lparser::bbox_z == 0)
@@ -1622,8 +1598,6 @@ public:
 					(*point)->position *= new_size;
 			}
 
-		polyhedron->material = m_material.interface();
-
 		return_val_if_fail(is_valid(*polyhedron), 0);
 		//assert_warning(is_solid(polyhedron));
 
@@ -1637,33 +1611,33 @@ public:
 
 	static k3d::iplugin_factory& get_factory()
 	{
-		static k3d::plugin_factory<k3d::document_plugin<l_parser_implementation>, k3d::interface_list<k3d::imesh_source > > factory(
+		static k3d::plugin_factory<k3d::document_plugin<l_parser>, k3d::interface_list<k3d::imesh_source > > factory(
 			k3d::uuid(0xa637e99d, 0x707c4342, 0x8c6d4d15, 0x78c9054a),
 			"LSystemParser",
-			"Generates a L-System object from a configuration file",
-			"Objects",
+			_("Generates an L-System object from a configuration file"),
+			"Polygons",
 			k3d::iplugin_factory::EXPERIMENTAL);
 
 		return factory;
 	}
 
 private:
-	k3d_data_property(boost::filesystem::path, k3d::immutable_name, k3d::change_signal, k3d::with_undo, k3d::local_storage, k3d::no_constraint) m_file_path;
-	k3d_measurement_property(double, k3d::immutable_name, k3d::change_signal, k3d::with_undo, k3d::local_storage, k3d::with_constraint) m_recursion;
-	k3d_measurement_property(double, k3d::immutable_name, k3d::change_signal, k3d::with_undo, k3d::local_storage, k3d::no_constraint) m_basic_angle;
-	k3d_measurement_property(double, k3d::immutable_name, k3d::change_signal, k3d::with_undo, k3d::local_storage, k3d::no_constraint) m_thickness;
-	k3d_measurement_property(unsigned long, k3d::immutable_name, k3d::change_signal, k3d::with_undo, k3d::local_storage, k3d::no_constraint) m_random_seed;
-	k3d_measurement_property(double, k3d::immutable_name, k3d::change_signal, k3d::with_undo, k3d::local_storage, k3d::no_constraint) m_size;
-	k3d_data_property(bool, k3d::immutable_name, k3d::change_signal, k3d::with_undo, k3d::local_storage, k3d::no_constraint) m_closed_form;
-	k3d_measurement_property(unsigned long, k3d::immutable_name, k3d::change_signal, k3d::with_undo, k3d::local_storage, k3d::no_constraint) m_mutations;
-	k3d_measurement_property(unsigned long, k3d::immutable_name, k3d::change_signal, k3d::with_undo, k3d::local_storage, k3d::no_constraint) m_mutation_seed;
-	k3d_measurement_property(unsigned long, k3d::immutable_name, k3d::change_signal, k3d::with_undo, k3d::local_storage, k3d::no_constraint) m_max_stack_size;
+	k3d_data(boost::filesystem::path, immutable_name, change_signal, with_undo, local_storage, no_constraint, path_property, path_serialization) m_file_path;
+	k3d_data(double, immutable_name, change_signal, with_undo, local_storage, with_constraint, measurement_property, with_serialization) m_recursion;
+	k3d_data(double, immutable_name, change_signal, with_undo, local_storage, no_constraint, measurement_property, with_serialization) m_basic_angle;
+	k3d_data(double, immutable_name, change_signal, with_undo, local_storage, no_constraint, measurement_property, with_serialization) m_thickness;
+	k3d_data(long, immutable_name, change_signal, with_undo, local_storage, no_constraint, measurement_property, with_serialization) m_random_seed;
+	k3d_data(double, immutable_name, change_signal, with_undo, local_storage, no_constraint, measurement_property, with_serialization) m_size;
+	k3d_data(bool, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_closed_form;
+	k3d_data(long, immutable_name, change_signal, with_undo, local_storage, no_constraint, measurement_property, with_serialization) m_mutations;
+	k3d_data(long, immutable_name, change_signal, with_undo, local_storage, no_constraint, measurement_property, with_serialization) m_mutation_seed;
+	k3d_data(long, immutable_name, change_signal, with_undo, local_storage, no_constraint, measurement_property, with_serialization) m_max_stack_size;
 };
 
 } // namespace libk3dlsystem
 
-K3D_MODULE_START(k3d::uuid(0xca6ffe7d, 0x5a7a4485, 0x979414de, 0xb8b1006a), Registry)
-	Registry.register_factory(libk3dlsystem::l_parser_implementation::get_factory());
+K3D_MODULE_START(libk3dlsystem, k3d::uuid(0xca6ffe7d, 0x5a7a4485, 0x979414de, 0xb8b1006a), Registry)
+	Registry.register_factory(libk3dlsystem::l_parser::get_factory());
 K3D_MODULE_END
 
 
