@@ -1,5 +1,5 @@
 // K-3D
-// Copyright (c) 1995-2004, Timothy M. Shead
+// Copyright (c) 1995-2005, Timothy M. Shead
 //
 // Contact: tshead@k-3d.com
 //
@@ -18,22 +18,21 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 /** \file
-		\brief Filter object that takes a mesh as input and returns the Catmull-Clark subdivision of this mesh.
+		\brief Modifier object that takes a mesh as input and returns the Catmull-Clark subdivision of this mesh.
 		\author Bart Janssens <bart.janssens@polytechnic.be>
 */
 
+#include <k3dsdk/i18n.h>
 #include <k3dsdk/imaterial.h>
-#include <k3dsdk/object.h>
-#include <k3dsdk/persistence.h>
 #include <k3dsdk/material.h>
-#include <k3dsdk/material_collection.h>
+#include <k3dsdk/material_client.h>
 #include <k3dsdk/measurement.h>
-#include <k3dsdk/mesh_filter.h>
+#include <k3dsdk/mesh_modifier.h>
 #include <k3dsdk/module.h>
-#include <k3dsdk/plugins.h>
-#include <k3dsdk/transform.h>
+#include <k3dsdk/node.h>
+#include <k3dsdk/persistent.h>
 
-#include "subdiv_algorithms.h"
+#include <subdivision_surface/k3d_sds_binding.h>
 
 #include <iterator>
 
@@ -44,21 +43,19 @@ namespace libk3dmesh
 // catmull_clark_subdivider_implementation
 
 class catmull_clark_subdivider_implementation :
-	public k3d::mesh_filter<k3d::persistent<k3d::object> >
+	public k3d::mesh_modifier<k3d::persistent<k3d::node> >
 {
-	typedef k3d::mesh_filter<k3d::persistent<k3d::object> > base;
+	typedef k3d::mesh_modifier<k3d::persistent<k3d::node> > base;
 
 public:
 	catmull_clark_subdivider_implementation(k3d::idocument& Document) :
 		base(Document),
-		m_levels(k3d::init_name("levels") + k3d::init_description("Levels [integer]") + k3d::init_constraint(k3d::constraint::minimum(1UL)) + k3d::init_document(Document) + k3d::init_value(1) + k3d::init_precision(0) + k3d::init_step_increment(1) + k3d::init_units(typeid(k3d::measurement::scalar)))
+		m_material(dynamic_cast<k3d::imaterial*>(k3d::default_material(Document))),
+		m_level(init_owner(*this) + init_name("level") + init_label(_("Level")) + init_description(_("Subdivision level")) + init_constraint(constraint::minimum(1L)) + init_value(1) + init_precision(0) + init_step_increment(1) + init_units(typeid(k3d::measurement::scalar)))
 	{
-		enable_serialization(k3d::persistence::proxy(m_levels));
-		register_property(m_levels);
-
-		m_input_mesh.changed_signal().connect(SigC::slot(*this, &catmull_clark_subdivider_implementation::on_reset_geometry));
-		m_levels.changed_signal().connect(SigC::slot(*this, &catmull_clark_subdivider_implementation::on_reset_geometry));
-		m_output_mesh.need_data_signal().connect(SigC::slot(*this, &catmull_clark_subdivider_implementation::on_create_geometry));
+		m_input_mesh.changed_signal().connect(sigc::mem_fun(*this, &catmull_clark_subdivider_implementation::on_reset_geometry));
+		m_level.changed_signal().connect(sigc::mem_fun(*this, &catmull_clark_subdivider_implementation::on_reset_geometry));
+		m_output_mesh.need_data_signal().connect(sigc::mem_fun(*this, &catmull_clark_subdivider_implementation::on_create_geometry));
 	}
 
 	void on_reset_geometry()
@@ -68,16 +65,20 @@ public:
 
 	k3d::mesh* on_create_geometry()
 	{
-		k3d::mesh* const input = m_input_mesh.property_value();
+		const k3d::mesh* input = m_input_mesh.value();
 		if(!input)
 			return 0;
 
-		const bool ignore_selection = !k3d::contains_selection(*input);
+		k3d::mesh* mesh(new k3d::mesh());
 
-		std::auto_ptr<k3d::mesh> mesh(new k3d::mesh());
+		// Set levels -before- input
+		m_sds_cache.set_levels(m_level.value());
+		m_sds_cache.set_input(input);
 
-		subdiv::catmull_clark(m_levels.property_value(), *input, *mesh, ignore_selection, true);
-		return mesh.release();
+		m_sds_cache.update();
+		m_sds_cache.output(mesh);
+
+		return mesh;
 	}
 
 	k3d::iplugin_factory& factory()
@@ -89,19 +90,22 @@ public:
 	{
 		static k3d::plugin_factory<
 			k3d::document_plugin<catmull_clark_subdivider_implementation>,
-				k3d::interface_list<k3d::imesh_source,
-				k3d::interface_list<k3d::imesh_sink > > > factory(
+			k3d::interface_list<k3d::imesh_source,
+			k3d::interface_list<k3d::imesh_sink > > > factory(
 				k3d::uuid(0xdc73f6e3, 0x744b4b51, 0xac535307, 0xc82d261c),
 				"CatmullClark",
-				"Generates a Catmull-Clark subdivision mesh",
-				"Objects",
+				_("Generates a Catmull-Clark subdivision mesh"),
+				"Subdivision",
 				k3d::iplugin_factory::STABLE);
 
 		return factory;
 	}
 
 private:
-	k3d_measurement_property(unsigned long, k3d::immutable_name, k3d::change_signal, k3d::with_undo, k3d::local_storage, k3d::with_constraint) m_levels;
+	k3d::imaterial* m_material;
+	k3d_data(long, immutable_name, change_signal, with_undo, local_storage, with_constraint, measurement_property, with_serialization) m_level;
+
+	k3d::sds::k3d_mesh_sds_cache m_sds_cache;
 };
 
 /////////////////////////////////////////////////////////////////////////////
@@ -109,9 +113,8 @@ private:
 
 k3d::iplugin_factory& catmull_clark_subdivision_factory()
 {
-	return catmull_clark_subdivider_implementation::get_factory();
+	return libk3dmesh::catmull_clark_subdivider_implementation::get_factory();
 }
 
 } // namespace libk3dmesh
 
-
