@@ -30,6 +30,9 @@
 namespace libk3dyafray
 {
 
+namespace yafray
+{
+	
 /// Abstract interface for YafRay light objects
 class ilight :
 	public virtual k3d::iunknown
@@ -44,6 +47,8 @@ protected:
 	virtual ~ilight() {}
 };
 
+} // namespace yafray
+
 } // namespace libk3dyafray
 
 #endif // !YAFRAY_ILIGHT_H
