@@ -30,6 +30,9 @@
 namespace libk3dyafray
 {
 
+namespace yafray
+{
+	
 /// Abstract interface for YafRay material objects
 class imaterial :
 	public virtual k3d::iunknown
@@ -44,6 +47,8 @@ protected:
 	virtual ~imaterial() {}
 };
 
+} // namespace yafray
+
 } // namespace libk3dyafray
 
 #endif // !YAFRAY_IMATERIAL_H
