@@ -1,7 +1,7 @@
 // K-3D
-// Copyright (c) 1995-2004, Timothy M. Shead
+// Copyright (c) 2004-2005, Romain Behar
 //
-// Contact: tshead@k-3d.com
+// Contact: romainbehar@yahoo.com
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public
@@ -25,28 +25,29 @@
 #include <k3dsdk/algebra.h>
 #include <k3dsdk/classes.h>
 #include <k3dsdk/color.h>
+#include <k3dsdk/create_plugins.h>
 #include <k3dsdk/file_helpers.h>
 #include <k3dsdk/idag.h>
 #include <k3dsdk/ideletable.h>
 #include <k3dsdk/idocument.h>
 #include <k3dsdk/idocument_plugin_factory.h>
 #include <k3dsdk/ifile_format.h>
-#include <k3dsdk/igeometry_read_format.h>
+#include <k3dsdk/idocument_read_format.h>
 #include <k3dsdk/imaterial.h>
-#include <k3dsdk/imaterial_collection.h>
 #include <k3dsdk/imesh_sink.h>
 #include <k3dsdk/imesh_source.h>
-#include <k3dsdk/iobject.h>
-#include <k3dsdk/iobject_collection.h>
+#include <k3dsdk/imesh_storage.h>
+#include <k3dsdk/inode.h>
+#include <k3dsdk/inode_collection.h>
 #include <k3dsdk/itransform_sink.h>
 #include <k3dsdk/itransform_source.h>
 #include <k3dsdk/material.h>
 #include <k3dsdk/mesh.h>
 #include <k3dsdk/module.h>
-#include <k3dsdk/plugins.h>
+#include <k3dsdk/nodes.h>
 #include <k3dsdk/property.h>
 #include <k3dsdk/result.h>
-#include <k3dsdk/selection.h>
+#include <k3dsdk/string_cast.h>
 #include <k3dsdk/string_modifiers.h>
 #include <k3dsdk/utility.h>
 #include <k3dsdk/vectors.h>
@@ -54,7 +55,7 @@
 #include <Hapy/Parser.h>
 #include <Hapy/Rules.h>
 
-#include <boost/filesystem/fstream.hpp>
+#include <k3dsdk/fstream.h>
 #include <boost/filesystem/path.hpp>
 
 #include <iostream>
@@ -135,52 +136,52 @@ const std::string no_quotes(const std::string& s)
 }
 
 // Creates a new mesh, put it in a FrozenMesh() and connect it a MeshInstance
-k3d::mesh* create_mesh_instance(k3d::idocument& Document, const std::string& Name, k3d::iobject*& ReturnFrozenMesh, k3d::iobject*& ReturnMeshInstance)
+k3d::mesh* create_mesh_instance(k3d::idocument& Document, const std::string& Name, k3d::inode*& ReturnFrozenMesh, k3d::inode*& ReturnMeshInstance)
 {
-	k3d::mesh* const mesh = new k3d::mesh();
-	return_val_if_fail(mesh, 0);
+	k3d::mesh* const new_mesh = new k3d::mesh();
+	return_val_if_fail(new_mesh, 0);
 
 	// Create document object ...
-	k3d::iobject* const frozen_mesh = k3d::create_document_plugin(k3d::classes::FrozenMesh(), Document);
+	k3d::inode* const frozen_mesh = k3d::create_plugin<k3d::inode>(k3d::classes::FrozenMesh(), Document);
 	return_val_if_fail(frozen_mesh, 0);
 	ReturnFrozenMesh = frozen_mesh;
 
-	k3d::imesh_sink* const frozen_mesh_sink = dynamic_cast<k3d::imesh_sink*>(frozen_mesh);
-	return_val_if_fail(frozen_mesh_sink, 0);
-	k3d::imesh_source* const frozen_mesh_source = dynamic_cast<k3d::imesh_source*>(frozen_mesh);
-	return_val_if_fail(frozen_mesh_source, 0);
-	k3d::set_property_value(frozen_mesh_sink->mesh_sink_input(), mesh);
+	k3d::imesh_storage* const frozen_mesh_storage = dynamic_cast<k3d::imesh_storage*>(frozen_mesh);
+	return_val_if_fail(frozen_mesh_storage, false);
 
-	frozen_mesh->set_name(Name);
+	frozen_mesh_storage->reset_mesh(new_mesh);
+	frozen_mesh->set_name(k3d::unique_name(Document.nodes(), Name));
 
 	// Create mesh object instance ...
-	k3d::iobject* instance = k3d::create_document_plugin(k3d::classes::MeshInstance(), Document);
+	k3d::inode* instance = k3d::create_plugin<k3d::inode>(k3d::classes::MeshInstance(), Document);
 	return_val_if_fail(instance, 0);
 	ReturnMeshInstance = instance;
 
-	instance->set_name(Name + " instance");
+	instance->set_name(k3d::unique_name(Document.nodes(), Name + " instance"));
 
 	// Set dependencies ...
 	k3d::imesh_sink* const instance_sink = dynamic_cast<k3d::imesh_sink*>(instance);
 	return_val_if_fail(instance_sink, false);
+	k3d::imesh_source* const frozen_mesh_source = dynamic_cast<k3d::imesh_source*>(frozen_mesh);
+	return_val_if_fail(frozen_mesh_source, 0);
 
 	k3d::idag::dependencies_t dependencies;
 	dependencies[&instance_sink->mesh_sink_input()] = &frozen_mesh_source->mesh_source_output();
 	Document.dag().set_dependencies(dependencies);
 
-	return mesh;
+	return new_mesh;
 }
 
 /////////////////////////////////////////////////////////////////////////////
-// yafray_reader_implementation
+// yafray_reader
 
-class yafray_reader_implementation :
+class yafray_reader :
 	public k3d::ifile_format,
-	public k3d::igeometry_read_format,
+	public k3d::idocument_read_format,
 	public k3d::ideletable
 {
 public:
-	yafray_reader_implementation()
+	yafray_reader()
 	{
 		m_inside_world = false;
 
@@ -206,16 +207,6 @@ public:
 		return "xml" == k3d::file_extension(FilePath);
 	}
 
-	bool pre_read(k3d::idocument& Document, const boost::filesystem::path& FilePath)
-	{
-		return true;
-	}
-
-	bool read_options(k3d::idocument& Document, const boost::filesystem::path& FilePath)
-	{
-		return true;
-	}
-
 	bool read_file(k3d::idocument& Document, const boost::filesystem::path& FilePath);
 
 	k3d::iplugin_factory& factory()
@@ -225,11 +216,11 @@ public:
 
 	static k3d::iplugin_factory& get_factory()
 	{
-		static k3d::plugin_factory<k3d::application_plugin<yafray_reader_implementation>, k3d::interface_list<k3d::igeometry_read_format> > factory(
+		static k3d::plugin_factory<k3d::application_plugin<yafray_reader>, k3d::interface_list<k3d::idocument_read_format> > factory(
 			k3d::uuid(0x872c46ea, 0x5b81454b, 0xb269f9db, 0x5184d7cd),
 			"YafrayReader",
 			"Yafray ( .xml )",
-			"");
+			"Yafray GeometryReader");
 
 		return factory;
 	}
@@ -265,19 +256,19 @@ private:
 	std::string m_solid_type;
 
 	// Mesh variables and functions
-	std::stack<k3d::iobject*> m_transform_objects;
+	std::stack<k3d::inode*> m_transform_objects;
 	k3d::mesh* m_current_mesh;
 	unsigned long m_current_mesh_points_index;
 	std::string m_current_object_name;
 	std::string m_current_name;
-	k3d::iobject* m_current_mesh_object;
-	k3d::iobject* m_current_mesh_instance;
+	k3d::inode* m_current_mesh_object;
+	k3d::inode* m_current_mesh_instance;
 	k3d::nupatch* m_current_nupatch;
 	k3d::bilinear_patch* m_current_bilinear_patch;
 	k3d::polyhedron* m_current_polyhedron;
 	k3d::linear_curve_group* m_current_linear_curve_group;
-	k3d::iobject* m_current_material;
-	k3d::iobject* m_current_shader;
+	k3d::inode* m_current_material;
+	k3d::inode* m_current_shader;
 
 	bool create_mesh(k3d::idocument& Document, const std::string& name)
 	{
@@ -312,16 +303,16 @@ private:
 	bool interpret(const Result& result, k3d::idocument& Document);
 	void load_light(const Hapy::Pree& Light, k3d::idocument& Document);
 
-	void set_current_transformations(k3d::iobject* Object)
+	void set_current_transformations(k3d::inode* Object)
 	{
 		if(!m_transformations.size())
 			return;
 
 		assert_warning(Object);
 
-		assert_warning(k3d::set_property_value(*Object, "position", k3d::vector3(extractTranslation(*m_transformations.rbegin()))));
-		assert_warning(k3d::set_property_value(*Object, "orientation", k3d::angle_axis(k3d::rotation3D(*m_transformations.rbegin()))));
-		assert_warning(k3d::set_property_value(*Object, "scale", k3d::vector3(k3d::extractScaling(*m_transformations.rbegin()))));
+		assert_warning(k3d::set_value(*Object, "position", k3d::vector3(extract_translation(*m_transformations.rbegin()))));
+		assert_warning(k3d::set_value(*Object, "orientation", k3d::angle_axis(k3d::rotation3D(*m_transformations.rbegin()))));
+		assert_warning(k3d::set_value(*Object, "scale", k3d::vector3(k3d::deprecated_extract_scaling(*m_transformations.rbegin()))));
 	}
 };
 
@@ -349,7 +340,7 @@ const std::string get_attribute(const Pree& tag, const std::string& name)
 	return std::string("");
 }
 
-void yafray_reader_implementation::parse_subpree(const Hapy::Pree& Node, k3d::idocument& Document)
+void yafray_reader::parse_subpree(const Hapy::Pree& Node, k3d::idocument& Document)
 {
 	const Pree& alt = Node[0];
 	if(alt.rid() == rOpenElement.id())
@@ -450,7 +441,7 @@ void yafray_reader_implementation::parse_subpree(const Hapy::Pree& Node, k3d::id
 							m_current_mesh->points.push_back(point);
 						}
 					else
-						std::cerr << warning << "unused 'p' element : " << alt.image() << std::endl;
+						k3d::log() << warning << "unused 'p' element : " << alt.image() << std::endl;
 				}
 			else if(name.image() == "f")
 				{
@@ -463,63 +454,62 @@ void yafray_reader_implementation::parse_subpree(const Hapy::Pree& Node, k3d::id
 							unsigned long c = k3d::from_string<unsigned long>(get_attribute(alt, "c"), 0);
 
 							// Create triangle
-							k3d::polyhedron::edges_t edges;
+							std::vector<k3d::split_edge*> edges;
 							edges.push_back(new k3d::split_edge(m_current_mesh->points[a + m_current_mesh_points_index]));
 							edges.push_back(new k3d::split_edge(m_current_mesh->points[b + m_current_mesh_points_index]));
 							edges.push_back(new k3d::split_edge(m_current_mesh->points[c + m_current_mesh_points_index]));
 
 							k3d::loop_edges(edges.begin(), edges.end());
-							m_current_polyhedron->edges.insert(m_current_polyhedron->edges.end(), edges.begin(), edges.end());
 
-							k3d::face* const face = new k3d::face(*edges.begin());
+							k3d::face* const face = new k3d::face(*edges.begin(), 0);
 							return_if_fail(face);
 
 							m_current_polyhedron->faces.push_back(face);
 						}
 					else
-						std::cerr << warning << "unused 'f' element : " << alt.image() << std::endl;
+						k3d::log() << warning << "unused 'f' element : " << alt.image() << std::endl;
 				}
 			else
-				std::cerr << warning << "unknown element : " << alt.image() << std::endl;
+				k3d::log() << warning << "unknown element : " << alt.image() << std::endl;
 		}
 	else
-		std::cerr << warning << "Unknown element" << std::endl;
+		k3d::log() << warning << "Unknown element" << std::endl;
 }
 
-void yafray_reader_implementation::load_light(const Hapy::Pree& Light, k3d::idocument& Document)
+void yafray_reader::load_light(const Hapy::Pree& Light, k3d::idocument& Document)
 {
 	const std::string type = get_attribute(Light, "type");
 	if(type == "hemilight")
 		{
-			k3d::iobject* const hemi_light = k3d::create_document_plugin(k3d::uuid(0xa0661dc7, 0x52cd4990, 0x8e6a0aa8, 0x87bdd89d), Document);
+			k3d::inode* const hemi_light = k3d::create_plugin<k3d::inode>(k3d::uuid(0xa0661dc7, 0x52cd4990, 0x8e6a0aa8, 0x87bdd89d), Document);
 
 			const std::string name = get_attribute(Light, "name");
 			hemi_light->set_name(name);
 
 			// Set properties
-			assert_warning(k3d::set_property_value(*hemi_light, "power", k3d::from_string<double>(get_attribute(Light, "power"), 0)));
-			assert_warning(k3d::set_property_value(*hemi_light, "samples", k3d::from_string<unsigned long>(get_attribute(Light, "samples"), 0)));
+			assert_warning(k3d::set_value(*hemi_light, "power", k3d::from_string<double>(get_attribute(Light, "power"), 0)));
+			assert_warning(k3d::set_value(*hemi_light, "samples", k3d::from_string<unsigned long>(get_attribute(Light, "samples"), 0)));
 		}
 	else if(type == "spotlight")
 		{
-			k3d::iobject* const spot_light = k3d::create_document_plugin(k3d::uuid(0x5e363371, 0xf8464895, 0x99f0ddf0, 0x4e26ee4a), Document);
+			k3d::inode* const spot_light = k3d::create_plugin<k3d::inode>(k3d::uuid(0x5e363371, 0xf8464895, 0x99f0ddf0, 0x4e26ee4a), Document);
 
 			const std::string name = get_attribute(Light, "name");
 			spot_light->set_name(name);
 
 			// Set properties
-			assert_warning(k3d::set_property_value(*spot_light, "power", k3d::from_string<double>(get_attribute(Light, "power"), 0)));
-			assert_warning(k3d::set_property_value(*spot_light, "size", k3d::from_string<double>(get_attribute(Light, "size"), 0)));
-			assert_warning(k3d::set_property_value(*spot_light, "nblend", k3d::from_string<double>(get_attribute(Light, "blend"), 0)));
-			assert_warning(k3d::set_property_value(*spot_light, "beam_falloff", k3d::from_string<double>(get_attribute(Light, "beam_falloff"), 0)));
+			assert_warning(k3d::set_value(*spot_light, "power", k3d::from_string<double>(get_attribute(Light, "power"), 0)));
+			assert_warning(k3d::set_value(*spot_light, "size", k3d::from_string<double>(get_attribute(Light, "size"), 0)));
+			assert_warning(k3d::set_value(*spot_light, "nblend", k3d::from_string<double>(get_attribute(Light, "blend"), 0)));
+			assert_warning(k3d::set_value(*spot_light, "beam_falloff", k3d::from_string<double>(get_attribute(Light, "beam_falloff"), 0)));
 		}
 	else
 		{
-			std::cerr << warning << "unknown light type : " << type << std::endl;
+			k3d::log() << warning << "unknown light type : " << type << std::endl;
 		}
 }
 
-bool yafray_reader_implementation::interpret(const Result& result, k3d::idocument& Document)
+bool yafray_reader::interpret(const Result& result, k3d::idocument& Document)
 {
 	if(result.statusCode == Result::scMatch)
 		{
@@ -528,20 +518,20 @@ bool yafray_reader_implementation::interpret(const Result& result, k3d::idocumen
 		}
 
 	if(result.input.size() > 0)
-		std::cerr << result.location() << ": syntax error" << std::endl;
+		k3d::log() << result.location() << ": syntax error" << std::endl;
 
 	return false;
 }
 
 
-bool yafray_reader_implementation::read_file(k3d::idocument& Document, const boost::filesystem::path& FilePath)
+bool yafray_reader::read_file(k3d::idocument& Document, const boost::filesystem::path& FilePath)
 {
 	// Try to open the input file ...
-	boost::filesystem::ifstream yafray_file(FilePath);
+	k3d::filesystem::ifstream yafray_file(FilePath);
 	if(!yafray_file.good())
 		{
-			std::cerr << warning << __PRETTY_FUNCTION__ << ": error opening [" << FilePath.native_file_string() << "]" << std::endl;
-			return_val_if_fail(0, false);
+			k3d::log() << error << "Error opening [" << FilePath.native_file_string() << "]" << std::endl;
+			return false;
 		}
 
 	// Parse file
@@ -589,7 +579,7 @@ if(yafray_file.eof())
 
 k3d::iplugin_factory& yafray_reader_factory()
 {
-	return yafray_reader_implementation::get_factory();
+	return yafray_reader::get_factory();
 }
 
 } // namespace libk3dyafray
